/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_ACTION_H_
#define _HWS_ACTION_H_

/* Max number of STEs needed for a rule (including match) */
#define ACTION_MAX_STE 10
#define WIRE_PORT 0xFFFF

enum action_stc_definition {
	ACTION_STC_MAX_ACTIONS_STC = 2,
	ACTION_STC_MAX_RESOURCES_FOR_ACTION = 3,
	ACTION_STC_ACTION_MAX_STC  = (ACTION_STC_MAX_ACTIONS_STC +
				      ACTION_STC_MAX_RESOURCES_FOR_ACTION),
};

enum action_type {
	ACTION_TYP_REFORMAT_TNL_L2_TO_L2,
	ACTION_TYP_REFORMAT_L2_TO_TNL_L2,
	ACTION_TYP_REFORMAT_TNL_L3_TO_L2,
	ACTION_TYP_REFORMAT_L2_TO_TNL_L3,
	ACTION_TYP_REFORMAT_TRAILER,
	ACTION_TYP_INSERT_HEADER,
	ACTION_TYP_REMOVE_HEADER,
	ACTION_TYP_CTR,
	ACTION_TYP_TAG,
	ACTION_TYP_MODIFY_HDR,
	ACTION_TYP_POP_VLAN,
	ACTION_TYP_PUSH_VLAN,
	ACTION_TYP_BARRIER,
	ACTION_TYP_ASO_METER,
	ACTION_TYP_ASO_CT,
	ACTION_TYP_ASO_IPSEC,
	ACTION_TYP_ASO_FIRST_HIT,
	ACTION_TYP_NAT64,
	ACTION_TYP_CRYPTO_ENCRYPT,
	ACTION_TYP_CRYPTO_DECRYPT,
	ACTION_TYP_DEST_ROOT,
	ACTION_TYP_DEST_ARRAY,
	ACTION_TYP_DEST_DROP,
	ACTION_TYP_DEST_TIR,
	ACTION_TYP_DEST_TBL,
	ACTION_TYP_DEST_VPORT,
	ACTION_TYP_DEST_MISS,
	ACTION_TYP_DEST_MATCHER,
	ACTION_TYP_INLINE_SET,
	ACTION_TYP_INLINE_ADD,
	ACTION_TYP_INLINE_ADD_FIELD,
	ACTION_TYP_INLINE_COPY,
	ACTION_TYP_INLINE_INSERT,
	ACTION_TYP_MAX,
};

enum action_offset {
	ACTION_OFFSET_DW0 = 0,
	ACTION_OFFSET_DW5 = 5,
	ACTION_OFFSET_DW6 = 6,
	ACTION_OFFSET_DW7 = 7,
	ACTION_OFFSET_HIT = 3,
	ACTION_OFFSET_HIT_LSB = 4,
};

enum action_stc_idx {
	ACTION_STC_IDX_CTRL = 0,
	ACTION_STC_IDX_HIT = 1,
	ACTION_STC_IDX_DW5 = 2,
	ACTION_STC_IDX_DW6 = 3,
	ACTION_STC_IDX_DW7 = 4,
	ACTION_STC_IDX_MAX = 5,
	/* STC Jumbo STE combo: CTR, Hit */
	ACTION_STC_IDX_LAST_JUMBO_STE = 1,
	/* STC combo1: CTR, SINGLE, DOUBLE, Hit */
	ACTION_STC_IDX_LAST_COMBO1 = 3,
	/* STC combo2: CTR, 3 x SINGLE, Hit */
	ACTION_STC_IDX_LAST_COMBO2 = 4,
	/* STC combo2: CTR, TRIPLE, Hit */
	ACTION_STC_IDX_LAST_COMBO3 = 2,
};

enum action_setter_flag {
	ASF_SINGLE1 = 1 << 0,
	ASF_SINGLE2 = 1 << 1,
	ASF_SINGLE3 = 1 << 2,
	ASF_DOUBLE = ASF_SINGLE2 | ASF_SINGLE3,
	ASF_TRIPLE = ASF_SINGLE1 | ASF_DOUBLE,
	ASF_INSERT = 1 << 3,
	ASF_REMOVE = 1 << 4,
	ASF_MODIFY = 1 << 5,
	ASF_CTR = 1 << 6,
};

enum action_mh {
	ACTION_MH_DOUBLE_SIZE = 8,
	ACTION_MH_INLINE_DATA_SIZE = 4,
	ACTION_MH_REFORMAT_DATA_SIZE = 64,
	ACTION_MH_DECAP_L3_NUM_ACTIONS_W_NO_VLAN = 6,
	ACTION_MH_DECAP_L3_NUM_ACTIONS_W_VLAN = 7,
};

enum action_header_length {
	ACTION_HDR_LEN_L2_MACS = 12,
	ACTION_HDR_LEN_L2_VLAN = 4,
	ACTION_HDR_LEN_L2_ETHER = 2,
	ACTION_HDR_LEN_L2 = (ACTION_HDR_LEN_L2_MACS +
			     ACTION_HDR_LEN_L2_ETHER),
	ACTION_HDR_LEN_L2_W_VLAN = (ACTION_HDR_LEN_L2 +
					ACTION_HDR_LEN_L2_VLAN),
};

enum action_aso_offsets {
	ACTION_ASO_OFFSET_METER_INIT_COLOR = 1,
	ACTION_ASO_OFFSET_FIRST_HIT_SET = 9,
};

enum action_nat64_details {
	ACTION_NAT64_MAX_MODIFY_ACTIONS = 20,
	/* IPv4 len is 20 bytes of the header + data, IPv6 len is only data, diff is 20 bytes */
	ACTION_NAT64_ADD_20 = 20,
	ACTION_NAT64_HEADER_MINUS_ONE = 9,
	ACTION_NAT64_IPV6_HEADER = 10,
	ACTION_NAT64_IPV4_HEADER_SIZE = 5,
	ACTION_NAT64_TTL_DEFAULT_VAL = 64,
	ACTION_NAT64_ECN_SIZE = 2,
	/* 2' comp to 20, to get -20 in add operation */
	ACTION_NAT64_DEC_20  = 0xffffffec,
	/* NAT64 data size per each stage, used for the metric calculations */
	ACTION_NAT64_COPY_DATA_SIZE = 88,
	ACTION_NAT64_REPLACE_DATA_SIZE = 96,
	ACTION_NAT64_COPY_PROTOCOL_DATA_SIZE = 16,
	ACTION_NAT64_COPYBACK_DATA_SIZE = 56,
};

/* 4 stages for the nat64 action */
enum action_nat64_stages {
	ACTION_NAT64_STAGE_COPY = 0,
	ACTION_NAT64_STAGE_REPLACE = 1,
	ACTION_NAT64_STAGE_COPY_PROTOCOL = 2,
	ACTION_NAT64_STAGE_COPYBACK = 3,
	/* Number of MH in NAT64 */
	ACTION_NAT64_STAGES = 4,
};

/* Registers for keeping data from stage to stage */
enum action_nat64_registers {
	ACTION_NAT64_REG_CONTROL = 0,
	ACTION_NAT64_REG_SRC_IP = 1,
	ACTION_NAT64_REG_DST_IP = 2,
	ACTION_NAT64_REG_MAX = 3,
};

struct actions_wqe_setter;
struct actions_apply_data;

typedef void (*action_setter_fp)
	(struct actions_apply_data *apply,
	 struct actions_wqe_setter *setter);

struct actions_wqe_setter {
	action_setter_fp set_single;
	action_setter_fp set_double;
	action_setter_fp set_triple;
	action_setter_fp set_hit;
	action_setter_fp set_ctr;
	uint8_t idx_single;
	uint8_t idx_double;
	uint8_t idx_triple;
	uint8_t idx_ctr;
	uint8_t idx_hit;
	uint8_t stage_idx;
	uint8_t flags;
	uint8_t extra_data;
};

struct actions_apply_data {
	struct send_engine *queue;
	struct mlx5dv_hws_action_data *action_data;
	struct mlx5dv_hws_action *dest_action;
	struct mlx5dv_hws_at *at;
	__be32 *wqe_data;
	struct send_wqe_gta_ctrl_seg *wqe_ctrl;
	uint32_t jump_to_action_stc;
	struct action_default_stc *default_stcs;
	enum mlx5dv_hws_table_type tbl_type;
	uint32_t next_direct_idx;
	uint8_t require_dep;
	struct mlx5dv_hws_rule *rule;
};

struct action_default_stc {
	struct pool_chunk nop_ctr;
	struct pool_chunk nop_dw5;
	struct pool_chunk nop_dw6;
	struct pool_chunk nop_dw7;
	struct pool_chunk default_hit;
	uint32_t refcount;
};

struct mlx5dv_hws_at {
	struct mlx5dv_hws_action *actions;
	uint8_t num_of_action_stes;
	uint8_t num_actions;
	uint8_t only_term;
	struct actions_wqe_setter setters[ACTION_MAX_STE + 1];
};

struct action_stc_cache {
	struct list_head head;
};

struct action_stc_cache_data {
	enum mlx5dv_hws_table_type tbl_type;
	uint32_t ib_port;
	struct cmd_stc_modify_attr attr;
};

struct action_stc_cache_entry {
	struct action_stc_cache_data data;
	struct pool_chunk stc;
	uint32_t refcount;
	struct list_node next;
};

struct mlx5dv_hws_action {
	enum action_type type;
	struct mlx5dv_hws_context *ctx;
	struct pool_chunk stc[ACTION_STC_ACTION_MAX_STC];
	uint8_t num_stc;
	uint64_t flags;
	union {
		struct {
			struct mlx5dv_devx_obj *devx_obj;
		} devx_dest;
		struct {
			uint16_t vport_num;
			uint16_t esw_owner_vhca_id;
		} vport;
		struct {
			size_t num_dest;
			struct cmd_set_fte_dest *dests;
			struct cmd_forward_tbl *fw_island;
		} dest_array;
		struct {
			struct mlx5dv_devx_obj *devx_obj;
			struct mlx5dv_hws_table *tbl;
		} dest_tbl;
		struct {
			struct mlx5dv_steering_anchor *sa;
		} root_tbl;
		struct {
			struct mlx5dv_hws_matcher *matcher;
		} dest_matcher;
		struct {
			struct mlx5dv_hws_resource **resource;
			uint8_t num_of_resources;
			size_t header_size;
		} reformat;
		struct {
			struct mlx5dv_hws_resource **resource;
			uint8_t num_of_resources;
			size_t num_of_actions;
			struct mlx5dv_devx_obj *pattern_obj;
			uint8_t reparse;
		} modify_header;
		struct {
			struct mlx5dv_hws_resource *resource;
			struct mlx5dv_devx_obj **pattern_obj;
			uint8_t need_reparse;
		} nat64;
		struct {
			struct mlx5dv_hws_resource **resource;
			size_t num_of_resources;
			uint8_t return_reg_id;
		} aso;
		struct {
			struct mlx5dv_hws_resource **resource;
			size_t num_of_resources;
		} counter;
		struct {
			struct mlx5dv_hws_resource **resource;
			size_t num_of_resources;
			size_t header_size;
			uint8_t anchor;
			uint8_t offset;
			uint8_t encap;
			uint8_t push_esp;
			uint8_t reparse;
		} insert_header;
		struct {
			uint8_t type;
			uint8_t start_anchor;
			uint8_t end_anchor;
			uint8_t num_of_words;
			uint8_t reparse;
			bool decap;
		} remove_header;
		struct {
			uint8_t type;
			uint8_t op;
			uint8_t size;
		} reformat_trailer;
		struct {
			__be64 stc_param;
			uint8_t reparse;
		} inline_modify_header;
		struct {
			struct mlx5dv_hws_resource **resource;
			size_t num_of_resources;
			enum mlx5dv_hws_action_crypto_type crypto_type;
		} crypto;
	};
	struct ibv_flow_action *flow_action;
};

static inline void
action_setter_default_single(struct actions_apply_data *apply,
			     struct actions_wqe_setter *setter)
{
	apply->wqe_data[ACTION_OFFSET_DW5] = 0;
	apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW5] =
		htobe32(apply->default_stcs->nop_dw5.offset);
}

static inline void
action_setter_default_double(struct actions_apply_data *apply,
			     struct actions_wqe_setter *setter)
{
	apply->wqe_data[ACTION_OFFSET_DW6] = 0;
	apply->wqe_data[ACTION_OFFSET_DW7] = 0;
	apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW6] =
		htobe32(apply->default_stcs->nop_dw6.offset);
	apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW7] =
		htobe32(apply->default_stcs->nop_dw7.offset);
}

static inline void
action_setter_default_ctr(struct actions_apply_data *apply,
			  struct actions_wqe_setter *setter)
{
	apply->wqe_data[ACTION_OFFSET_DW0] = 0;
	apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_CTRL] =
		htobe32(apply->default_stcs->nop_ctr.offset);
}

static inline void
action_apply_setter(struct actions_apply_data *apply,
		    struct actions_wqe_setter *setter,
		    bool is_jumbo)
{
	uint8_t num_of_actions;

	/* Set control counter */
	if (setter->set_ctr)
		setter->set_ctr(apply, setter);
	else
		action_setter_default_ctr(apply, setter);

	if (!is_jumbo) {
		if (unlikely(setter->set_triple)) {
			/* Set triple on match */
			setter->set_triple(apply, setter);
			num_of_actions = ACTION_STC_IDX_LAST_COMBO3;
		} else {
			/* Set single and double on match */
			if (setter->set_single)
				setter->set_single(apply, setter);
			else
				action_setter_default_single(apply, setter);

			if (setter->set_double)
				setter->set_double(apply, setter);
			else
				action_setter_default_double(apply, setter);

			num_of_actions = setter->set_double ?
				ACTION_STC_IDX_LAST_COMBO1 :
				ACTION_STC_IDX_LAST_COMBO2;
		}
	} else {
		apply->wqe_data[ACTION_OFFSET_DW5] = 0;
		apply->wqe_data[ACTION_OFFSET_DW6] = 0;
		apply->wqe_data[ACTION_OFFSET_DW7] = 0;
		apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW5] = 0;
		apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW6] = 0;
		apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_DW7] = 0;
		num_of_actions = ACTION_STC_IDX_LAST_JUMBO_STE;
	}

	/* Set next/final hit action */
	setter->set_hit(apply, setter);

	/* Set number of actions */
	apply->wqe_ctrl->stc_ix[ACTION_STC_IDX_CTRL] |=
		htobe32(num_of_actions << 29);
}

int action_get_default_stc(struct mlx5dv_hws_context *ctx,
			   uint8_t tbl_type,
			   uint32_t ib_port);

void action_put_default_stc(struct mlx5dv_hws_context *ctx,
			    uint8_t tbl_type,
			    uint32_t ib_port);

int action_alloc_single_stc(struct mlx5dv_hws_context *ctx,
			    struct cmd_stc_modify_attr *stc_attr,
			    enum mlx5dv_hws_table_type type,
			    uint32_t ib_port,
			    struct pool_chunk *stc);

void action_free_single_stc(struct mlx5dv_hws_context *ctx,
			    uint32_t table_type,
			    uint32_t ib_port,
			    struct pool_chunk *stc);

int action_stc_cache_init(struct action_stc_cache **cache);

void action_stc_cache_uninit(struct action_stc_cache *cache);

int action_root_build_attr(struct mlx5dv_hws_action_data actions_data[],
			   struct mlx5dv_hws_at *at,
			   struct mlx5dv_hws_action *dest_action,
			   struct mlx5dv_flow_action_attr *attr);

int action_template_verify_actions(struct mlx5dv_hws_matcher *matcher,
				   struct mlx5dv_hws_at *at);

int action_template_bind(struct mlx5dv_hws_matcher *matcher,
				    struct mlx5dv_hws_at *at);

void action_template_unbind(struct mlx5dv_hws_matcher *matcher,
				       struct mlx5dv_hws_at *at);
#endif /* _HWS_ACTION_H_ */
