/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_CMD_H_
#define _HWS_CMD_H_

enum cmd_graph_node_max {
	/* Maximal number of samples per graph node. */
	CMD_GRAPH_NODE_MAX_SAMPLE_NUM = 8,
	/* Maximal number of input/output arcs per graph node. */
	CMD_GRAPH_NODE_MAX_ARC_NUM = 8,
};

enum cmd_ext_dest_flags {
	CMD_EXT_DEST_REFORMAT = 1 << 0,
	CMD_EXT_DEST_ESW_OWNER_VHCA_ID = 1 << 1,
};

struct cmd_set_fte_dest {
	uint8_t destination_type;
	uint32_t destination_id;
	enum cmd_ext_dest_flags ext_flags;
	struct mlx5dv_devx_obj *ext_reformat;
	size_t ext_reformat_hdr_sz;
	uint16_t esw_owner_vhca_id;
};

struct cmd_set_fte_attr {
	uint32_t action_flags;
	bool ignore_flow_level;
	uint8_t flow_source;
	uint8_t extended_dest:1;
	uint8_t uplink_hairpin_en:1;
	uint32_t packet_reformat_id;
	uint32_t dests_num;
	struct cmd_set_fte_dest *dests;
};

struct cmd_ft_create_attr {
	uint8_t type;
	uint8_t level;
	bool other_vport;
	uint16_t vport;
	bool rtc_valid;
	bool reformat_en;
};

struct cmd_ft_modify_attr {
	uint8_t type;
	uint32_t rtc_id_0;
	uint32_t rtc_id_1;
	uint32_t table_miss_id;
	uint8_t table_miss_action;
	uint64_t modify_fs;
	bool other_vport;
	uint16_t vport;
};

struct cmd_ft_query_attr {
	uint8_t type;
};

struct cmd_fg_attr {
	uint32_t table_id;
	uint32_t table_type;
};

struct cmd_forward_tbl {
	struct mlx5dv_devx_obj *ft;
	struct mlx5dv_devx_obj *fg;
	struct mlx5dv_devx_obj *fte;
	uint32_t refcount;
};

struct cmd_rtc_create_attr {
	uint32_t pd;
	uint32_t stc_base;
	uint32_t ste_base;
	uint32_t ste_offset;
	uint32_t miss_ft_id;
	bool fw_gen_wqe;
	uint8_t update_index_mode;
	uint8_t access_index_mode;
	uint8_t num_hash_definer;
	uint8_t log_depth;
	uint8_t log_size;
	uint8_t table_type;
	uint8_t match_definer_0;
	uint8_t match_definer_1;
	uint8_t reparse_mode;
	bool other_vport;
	uint16_t vport;
	bool is_frst_jumbo;
	bool is_scnd_range;
	bool is_scnd_compare;
	bool is_compare;
};

struct cmd_alias_obj_create_attr {
	uint32_t obj_id;
	uint16_t vhca_id;
	uint16_t obj_type;
	uint8_t access_key[MLX5_ACCESS_KEY_LEN];
};

struct cmd_stc_create_attr {
	uint8_t log_obj_range;
	uint8_t table_type;
	bool other_vport;
	uint16_t vport;
};

struct cmd_stc_modify_attr {
	uint32_t stc_offset;
	uint8_t action_offset;
	uint8_t reparse_mode;
	enum mlx5_ifc_stc_action_type action_type;
	union {
		uint32_t id; /* TIRN, TAG, FT ID, STE ID, CRYPTO */
		struct {
			uint8_t decap;
			uint16_t start_anchor;
			uint16_t end_anchor;
		} remove_header;
		struct {
			uint32_t arg_id;
			uint32_t pattern_id;
		} modify_header;
		struct {
			__be64 data;
		} modify_action;
		struct {
			uint32_t arg_id;
			uint32_t header_size;
			uint8_t is_inline;
			uint8_t encap;
			uint16_t insert_anchor;
			uint16_t insert_offset;
			uint8_t push_esp;
		} insert_header;
		struct {
			uint8_t aso_type;
			uint32_t devx_obj_id;
			uint8_t return_reg_id;
		} aso;
		struct {
			uint16_t vport_num;
			uint16_t esw_owner_vhca_id;
			uint8_t eswitch_owner_vhca_id_valid;
		} vport;
		struct {
			uint32_t ste_obj_id;
			uint32_t match_definer_id;
			uint8_t log_hash_size;
			bool ignore_tx;
		} ste_table;
		struct {
			uint16_t start_anchor;
			uint16_t num_of_words;
		} remove_words;
		struct {
			uint8_t type;
			uint8_t op;
			uint8_t size;
		} reformat_trailer;

		uint32_t dest_table_id;
		uint32_t dest_tir_num;
	};
};

struct cmd_ste_create_attr {
	uint8_t log_obj_range;
	uint8_t table_type;
	bool other_vport;
	uint16_t vport;
};

struct cmd_definer_create_attr {
	uint8_t *dw_selector;
	uint8_t *byte_selector;
	uint8_t *match_mask;
};

struct cmd_sq_create_attr {
	uint32_t cqn;
	uint32_t pdn;
	uint32_t page_id;
	uint32_t dbr_id;
	uint32_t wq_id;
	uint32_t log_wq_sz;
	uint32_t ts_format;
};

struct cmd_allow_other_vhca_access_attr {
	uint16_t obj_type;
	uint32_t obj_id;
	uint8_t access_key[MLX5_ACCESS_KEY_LEN];
};

struct cmd_packet_reformat_create_attr {
	uint8_t type;
	size_t data_sz;
	void *data;
	uint8_t reformat_param_0;
};

struct cmd_query_ft_caps {
	uint8_t max_level;
	uint8_t reparse:1;
	uint8_t ignore_flow_level_rtc_valid:1;
};

struct cmd_query_caps {
	uint32_t flex_protocols;
	uint8_t wqe_based_update;
	uint8_t rtc_reparse_mode;
	uint16_t ste_format;
	uint8_t rtc_index_mode;
	uint8_t ste_alloc_log_max;
	uint8_t ste_alloc_log_gran;
	uint8_t stc_alloc_log_max;
	uint8_t stc_alloc_log_gran;
	uint8_t rtc_log_depth_max;
	uint8_t format_select_gtpu_dw_0;
	uint8_t format_select_gtpu_dw_1;
	uint8_t flow_table_hash_type;
	uint8_t format_select_gtpu_dw_2;
	uint8_t format_select_gtpu_ext_dw_0;
	uint8_t access_index_mode;
	uint32_t linear_match_definer;
	bool full_dw_jumbo_support;
	bool rtc_hash_split_table;
	bool rtc_linear_lookup_table;
	uint8_t flow_counter_access_aso_opc_mod;
	uint32_t supp_type_gen_wqe;
	uint8_t rtc_max_hash_def_gen_wqe;
	uint8_t stc_max_trailer;
	uint16_t supp_ste_format_gen_wqe;
	struct cmd_query_ft_caps nic_ft;
	struct cmd_query_ft_caps fdb_ft;
	bool eswitch_manager;
	bool merged_eswitch;
	uint32_t eswitch_manager_vport_number;
	uint8_t log_header_modify_argument_granularity;
	uint8_t log_header_modify_argument_max_alloc;
	uint8_t sq_ts_format;
	uint8_t fdb_tir_stc:1;
	uint8_t stc_push_esp:1;
	uint8_t stc_jump_to_fdb_rx:1;
	uint8_t fdb_uplink_hairpin:1;
	uint64_t definer_format_sup;
	uint32_t trivial_match_definer;
	uint32_t vhca_id;
	bool cross_vhca_resources;
	uint32_t shared_vhca_id;
	char fw_ver[64];
	bool ipsec_offload;
	uint8_t encap_entropy_hash_type;
	bool roce;
	uint16_t roce_max_src_udp_port;
	uint16_t roce_min_src_udp_port;
	bool fdb_unified_en;
	uint8_t flow_counter_access_aso;
	uint64_t stc_action_type_63_0;
	uint64_t stc_action_type_127_64;
	uint32_t parse_graph_header_length_field_mask_width;
	uint32_t parse_graph_tlv_option_length_field_mask_width;
	uint32_t num_ib_ports;
	uint8_t flex_parser_id_geneve_opt_0;
	uint8_t geneve_tlv_opt;
};

struct cmd_generate_wqe_attr {
	uint8_t *wqe_ctrl;
	uint8_t *gta_ctrl;
	uint8_t *gta_data_0;
	uint8_t *gta_data_1;
	uint32_t pdn;
};

struct cmd_flex_sample_info_query {
	uint16_t modify_field_id;
	uint8_t field_format_select_dw;
	uint8_t ok_bit_format_select_dw;
	uint8_t ok_bit_offset;
};

int cmd_destroy_obj(struct mlx5dv_devx_obj *devx_obj);

struct mlx5dv_devx_obj *
cmd_flow_table_create(struct ibv_context *ctx,
		      struct cmd_ft_create_attr *ft_attr);

int cmd_flow_table_modify(struct mlx5dv_devx_obj *devx_obj,
			  struct cmd_ft_modify_attr *ft_attr);

int cmd_flow_table_modify_rtc(struct mlx5dv_devx_obj *ft,
			      uint32_t fw_ft_type,
			      bool other_vport,
			      uint16_t vport,
			      struct mlx5dv_devx_obj *rtc_0,
			      struct mlx5dv_devx_obj *rtc_1);

int cmd_flow_table_query(struct mlx5dv_devx_obj *devx_obj,
			 struct cmd_ft_query_attr *ft_attr,
			 uint64_t *icm_addr_0,
			 uint64_t *icm_addr_1);

struct mlx5dv_devx_obj *
cmd_rtc_create(struct ibv_context *ctx,
	       struct cmd_rtc_create_attr *rtc_attr);

struct mlx5dv_devx_obj *
cmd_stc_create(struct ibv_context *ctx,
	       struct cmd_stc_create_attr *stc_attr);

int cmd_stc_modify(struct mlx5dv_devx_obj *devx_obj,
		   struct cmd_stc_modify_attr *stc_attr);

struct mlx5dv_devx_obj *
cmd_ste_create(struct ibv_context *ctx,
	       struct cmd_ste_create_attr *ste_attr);

struct mlx5dv_devx_obj *
cmd_definer_create(struct ibv_context *ctx,
		   struct cmd_definer_create_attr *def_attr);

struct mlx5dv_devx_obj *
cmd_sq_create(struct ibv_context *ctx,
	      struct cmd_sq_create_attr *attr);

struct mlx5dv_devx_obj *
cmd_arg_create(struct ibv_context *ctx,
	       uint16_t log_obj_range,
	       uint32_t pdn);

struct mlx5dv_devx_obj *
cmd_flow_counter_create(struct ibv_context *ctx,
			uint16_t log_obj_size,
			uint32_t pdn);

struct mlx5dv_devx_obj *
cmd_conn_track_create(struct ibv_context *ctx,
		      uint16_t log_obj_size,
		      uint32_t pdn);

struct mlx5dv_devx_obj *
cmd_flow_hit_create(struct ibv_context *ctx,
		    uint16_t log_obj_size,
		    uint32_t pdn);

struct mlx5dv_devx_obj *
cmd_flow_meter_create(struct ibv_context *ctx,
		      uint16_t log_obj_size,
		      uint32_t pdn);

struct mlx5dv_devx_obj *
cmd_header_modify_pattern_create(struct ibv_context *ctx,
				 uint32_t pattern_length,
				 uint8_t *actions);

struct mlx5dv_devx_obj *
cmd_packet_reformat_create(struct ibv_context *ctx,
			   struct cmd_packet_reformat_create_attr *attr);

struct mlx5dv_devx_obj *
cmd_set_fte(struct ibv_context *ctx,
	    uint32_t table_type,
	    uint32_t table_id,
	    uint32_t group_id,
	    struct cmd_set_fte_attr *fte_attr);

struct cmd_forward_tbl *
cmd_forward_tbl_create(struct ibv_context *ctx,
		       struct cmd_ft_create_attr *ft_attr,
		       struct cmd_set_fte_attr *fte_attr);

void cmd_forward_tbl_destroy(struct cmd_forward_tbl *tbl);

struct mlx5dv_devx_obj *
cmd_alias_obj_create(struct ibv_context *ctx,
		     struct cmd_alias_obj_create_attr *alias_attr);

struct mlx5dv_devx_obj *
cmd_allow_and_create_aliased_obj(struct ibv_context *ibv_owner,
				 struct ibv_context *ibv_allowed,
				 uint16_t vhca_id_to_be_accessed,
				 uint32_t aliased_object_id,
				 uint16_t object_type);

int cmd_sq_modify_rdy(struct mlx5dv_devx_obj *devx_obj);

int cmd_ib_port_query(struct ibv_context *ctx,
		      uint32_t port_num,
		      uint16_t *vport_num,
		      uint16_t *esw_vhca_id);

int cmd_query_caps(struct ibv_context *ctx,
		   struct cmd_query_caps *caps);

void cmd_set_attr_connect_miss_tbl(struct mlx5dv_hws_context *ctx,
				   uint32_t fw_ft_type,
				   enum mlx5dv_hws_table_type type,
				   bool other_vport,
				   uint16_t vport,
				   struct cmd_ft_modify_attr *ft_attr);

int cmd_allow_other_vhca_access(struct ibv_context *ctx,
				struct cmd_allow_other_vhca_access_attr *attr);

int cmd_generate_wqe(struct ibv_context *ctx,
		     struct cmd_generate_wqe_attr *attr,
		     struct mlx5_cqe64 *ret_cqe);

int cmd_flex_sample_info_query(struct ibv_context *ctx,
			       uint32_t sampler_id,
			       struct cmd_flex_sample_info_query *info_query);

struct mlx5dv_devx_obj *cmd_flex_node_create(struct mlx5dv_hws_context *ctx,
					     struct mlx5dv_hws_parser_node *node);

struct mlx5dv_devx_obj *
cmd_geneve_tlv_option_create(struct ibv_context *ctx,
			     uint16_t option_class,
			     uint8_t option_type,
			     uint8_t option_data_len,
			     uint8_t option_class_ignore,
			     uint8_t offset_valid,
			     uint8_t sample_offset);

int
cmd_geneve_tlv_option_query(struct ibv_context *ctx,
			    struct mlx5dv_devx_obj *devx_obj,
			    uint32_t *sampler_id);

#endif /* _HWS_CMD_H_ */
