/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_CONTEXT_H_
#define _HWS_CONTEXT_H_

enum context_flags {
	CONTEXT_FLAG_HWS_SUPPORT = 1 << 0,
};

struct context_shared_gvmi_res {
	struct mlx5dv_devx_obj *end_ft;
	struct mlx5dv_devx_obj *aliased_end_ft;
	uint32_t refcount;
};

struct context_common_res {
	struct action_default_stc *default_stc;
	struct cmd_forward_tbl *default_miss;
	struct context_shared_gvmi_res *gvmi_res;
};

struct context_obj {
	void *obj;
	struct list_node next;
	enum mlx5dv_hws_table_type tbl_type;
	uint32_t ib_port;
	uint32_t refcount;
};

struct mlx5dv_hws_context {
	/* Datapath */
	struct send_engine *send_queue;
	size_t queues;
	/* Control */
	struct ibv_context *ibv_ctx;
	/* When local_ibv_ctx is not NULL shared_ibv is used for resources */
	struct ibv_context *local_ibv_ctx;
	struct cmd_query_caps *caps;
	uint32_t pd_num;
	struct pattern_cache *pattern_cache;
	struct definer_cache *definer_cache;
	struct action_stc_cache *stc_cache;
	pthread_spinlock_t ctrl_lock;
	pthread_spinlock_t stc_lock;
	enum context_flags flags;
	struct list_head head;
	struct list_head common_res;
	struct list_head stc_pool;
};

static inline bool context_shared_gvmi_used(struct mlx5dv_hws_context *ctx)
{
	return ctx->local_ibv_ctx ? true : false;
}

static inline struct ibv_context *
context_get_local_ibv(struct mlx5dv_hws_context *ctx)
{
	if (context_shared_gvmi_used(ctx))
		return ctx->local_ibv_ctx;

	return ctx->ibv_ctx;
}

struct context_common_res *
context_get_common_res(struct mlx5dv_hws_context *ctx,
		       enum mlx5dv_hws_table_type tbl_type,
		       uint32_t ib_port);

struct pool *
context_get_stc_pool(struct mlx5dv_hws_context *ctx,
		     enum mlx5dv_hws_table_type tbl_type,
		     uint32_t ib_port);

uint8_t context_get_reparse_mode(struct mlx5dv_hws_context *ctx);

bool context_cap_dynamic_reparse(struct mlx5dv_hws_context *ctx);

int context_query_ib_port(struct mlx5dv_hws_context *ctx,
			  uint32_t ib_port,
			  uint16_t *vport_num,
			  bool *other_vport);

#endif /* _HWS_CONTEXT_H_ */
