/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_DEBUG_H_
#define _HWS_DEBUG_H_

#define DEBUG_VERSION "1.0.MLX5DV_HWS"

#define HWS_LOG(arg...) \
	mlx5dv_hws_log_print(MLX5DV_HWS_DEBUG_LEVEL_LOG, ##arg)

#define HWS_DBG(arg...) \
	mlx5dv_hws_log_print(MLX5DV_HWS_DEBUG_LEVEL_DBG, ##arg)

#define HWS_ERR(arg...) \
	mlx5dv_hws_log_print(MLX5DV_HWS_DEBUG_LEVEL_ERR, ##arg)

enum mlx5dv_hws_debug_level {
	MLX5DV_HWS_DEBUG_LEVEL_ERR,
	MLX5DV_HWS_DEBUG_LEVEL_LOG,
	MLX5DV_HWS_DEBUG_LEVEL_DBG,
};

#define mlx5dv_hws_log_print(level, format, arg...) \
	fprintf(stderr, "MLX5DV_HWS[%s:%d]: " format "\n", __func__, __LINE__, ##arg)

enum debug_res_type {
	DEBUG_RES_TYPE_CONTEXT = 4000,
	DEBUG_RES_TYPE_CONTEXT_ATTR = 4001,
	DEBUG_RES_TYPE_CONTEXT_CAPS = 4002,
	DEBUG_RES_TYPE_CONTEXT_SEND_ENGINE = 4003,
	DEBUG_RES_TYPE_CONTEXT_SEND_RING = 4004,
	DEBUG_RES_TYPE_CONTEXT_STC = 4005,

	DEBUG_RES_TYPE_TABLE = 4100,

	DEBUG_RES_TYPE_MATCHER = 4200,
	DEBUG_RES_TYPE_MATCHER_ATTR = 4201,
	DEBUG_RES_TYPE_MATCHER_MATCH_TEMPLATE = 4202,
	DEBUG_RES_TYPE_MATCHER_TEMPLATE_MATCH_DEFINER = 4203,
	DEBUG_RES_TYPE_MATCHER_ACTION_TEMPLATE = 4204,
	DEBUG_RES_TYPE_MATCHER_TEMPLATE_HASH_DEFINER = 4205,
	DEBUG_RES_TYPE_MATCHER_TEMPLATE_RANGE_DEFINER = 4206,
	DEBUG_RES_TYPE_MATCHER_TEMPLATE_COMPARE_MATCH_DEFINER = 4207,
	DEBUG_RES_TYPE_MATCHER_RESIZABLE_ACTION_ARRAY = 4208,
};

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
#endif

const char *debug_action_type_to_str(enum action_type type);

static inline uint64_t
debug_icm_to_idx(uint64_t icm_addr)
{
	return (icm_addr >> 6) & 0xffffffff;
}

#endif /* _HWS_DEBUG_H_ */
