/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_DEFINER_H_
#define _HWS_DEFINER_H_

/* network protocols defines */
#define IPV6_VERSION_BE 0x60000000
#define IPV4_VER_IHL_BE 0x45000000
#define IPV4_ETHERTYPE 0x0800
#define IPV6_ETHERTYPE 0x86DD

/* Max available selecotrs */
#define DW_SELECTORS 9
#define BYTE_SELECTORS 8

/* Selectors based on match TAG */
#define DW_SELECTORS_MATCH 6
#define DW_SELECTORS_LIMITED 3

/* Selectors based on range TAG */
#define DW_SELECTORS_RANGE 2
#define BYTE_SELECTORS_RANGE 8

enum definer_dw_selectors {
	DEFINER_SELECTOR_DW0,
	DEFINER_SELECTOR_DW1,
	DEFINER_SELECTOR_DW2,
	DEFINER_SELECTOR_DW3,
	DEFINER_SELECTOR_DW4,
	DEFINER_SELECTOR_DW5,
	DEFINER_SELECTOR_DW6,
	DEFINER_SELECTOR_DW7,
	DEFINER_SELECTOR_DW8,
};

enum definer_type {
	DEFINER_TYPE_MATCH,
	DEFINER_TYPE_JUMBO,
	DEFINER_TYPE_RANGE,
	DEFINER_TYPE_COMPARE,
};

struct mlx5_ifc_header_gre_bits {
	union {
		u8 c_rsvd0_ver[0x10];
		struct {
			u8 gre_c_present[0x1];
			u8 reserved_at_1[0x1];
			u8 gre_k_present[0x1];
			u8 gre_s_present[0x1];
			u8 reserved_at_4[0x9];
			u8 version[0x3];
		};
	};
	u8 protocol[0x10];
	u8 checksum[0x10];
	u8 reserved_at_30[0x10];
	u8 key[0x20];
	u8 sequence_num[0x20];
};

struct mlx5_ifc_header_nvgre_bits {
	u8 flags_rsvd0_ver[0x10];
	u8 protocol[0x10];
	u8 vsid[0x18];
	u8 flow_id[0x8];
};

struct mlx5_ifc_header_vxlan_bits {
	u8 flags[0x8];
	u8 reserved1[0x18];
	u8 vni[0x18];
	u8 reserved2[0x8];
};

struct mlx5_ifc_header_vxlan_gpe_bits {
	u8 flags[0x8];
	u8 rsvd0[0x10];
	u8 protocol[0x8];
	u8 vni[0x18];
	u8 rsvd1[0x8];
};

struct mlx5_ifc_header_vxlan_gbp_bits {
	u8 flags[0x10];
	u8 group_id[0x10];
	u8 vni[0x18];
	u8 rsvd1[0x8];
};

enum definer_gtp {
	DEFINER_GTP_EXT_HDR_BIT = 0x04,
};

struct mlx5_ifc_header_gtp_bits {
	u8 version[0x3];
	u8 proto_type[0x1];
	u8 reserved1[0x1];
	u8 ext_hdr_flag[0x1];
	u8 seq_num_flag[0x1];
	u8 pdu_flag[0x1];
	u8 msg_type[0x8];
	u8 msg_len[0x10];
	u8 teid[0x20];
};

struct mlx5_ifc_header_opt_gtp_bits {
	u8 seq_num[0x10];
	u8 pdu_num[0x8];
	u8 next_ext_hdr_type[0x8];
};

struct mlx5_ifc_header_gtp_psc_bits {
	u8 len[0x8];
	u8 pdu_type[0x4];
	u8 flags[0x4];
	u8 qfi[0x8];
	u8 reserved2[0x8];
};

struct mlx5_ifc_header_ipv6_vtc_bits {
	u8 version[0x4];
	union {
		u8 tos[0x8];
		struct {
			u8 dscp[0x6];
			u8 ecn[0x2];
		};
	};
	u8 flow_label[0x14];
};

struct mlx5_ifc_header_ipv6_routing_ext_bits {
	u8 next_hdr[0x8];
	u8 hdr_len[0x8];
	u8 type[0x8];
	u8 segments_left[0x8];
	union {
		u8 flags[0x20];
		struct {
			u8 last_entry[0x8];
			u8 flag[0x8];
			u8 tag[0x10];
		};
	};
};

struct mlx5_ifc_header_geneve_bits {
	union {
		u8 ver_opt_len_o_c_rsvd[0x10];
		struct {
			u8 version[0x2];
			u8 opt_len[0x6];
			u8 o_flag[0x1];
			u8 c_flag[0x1];
			u8 reserved_at_a[0x6];
		};
	};
	u8 protocol_type[0x10];
	u8 vni[0x18];
	u8 reserved_at_38[0x8];
};

struct mlx5_ifc_header_geneve_opt_bits {
	u8 class[0x10];
	u8 type[0x8];
	u8 reserved[0x3];
	u8 len[0x5];
};

struct mlx5_ifc_header_icmp_bits {
	union {
		u8 icmp_dw1[0x20];
		struct {
			u8 type[0x8];
			u8 code[0x8];
			u8 cksum[0x10];
		};
	};
	union {
		u8 icmp_dw2[0x20];
		struct {
			u8 ident[0x10];
			u8 seq_nb[0x10];
		};
	};
};

struct mlx5_ifc_definer_hl_eth_l2_bits {
	u8 dmac_47_16[0x20];
	u8 dmac_15_0[0x10];
	u8 l3_ethertype[0x10];
	u8 reserved_at_40[0x1];
	u8 sx_sniffer[0x1];
	u8 functional_lb[0x1];
	u8 ip_fragmented[0x1];
	u8 qp_type[0x2];
	u8 encap_type[0x2];
	u8 port_number[0x2];
	u8 l3_type[0x2];
	u8 l4_type_bwc[0x2];
	union {
		u8 first_vlan_qualifier[0x2];
		struct {
			u8 first_cvlan_tag[0x1];
			u8 first_svlan_tag[0x1];
		};
	};
	u8 first_priority[0x3];
	u8 first_cfi[0x1];
	u8 first_vlan_id[0xc];
	u8 l4_type[0x4];
	u8 reserved_at_64[0x2];
	u8 ipsec_layer[0x2];
	u8 l2_type[0x2];
	u8 force_lb[0x1];
	u8 l2_ok[0x1];
	u8 l3_ok[0x1];
	u8 l4_ok[0x1];
	union {
		u8 second_vlan_qualifier[0x2];
		struct {
			u8 second_cvlan_tag[0x1];
			u8 second_svlan_tag[0x1];
		};
	};
	u8 second_priority[0x3];
	u8 second_cfi[0x1];
	u8 second_vlan_id[0xc];
};

struct mlx5_ifc_definer_hl_eth_l2_src_bits {
	u8 smac_47_16[0x20];
	u8 smac_15_0[0x10];
	u8 loopback_syndrome[0x8];
	u8 l3_type[0x2];
	u8 l4_type_bwc[0x2];
	u8 first_vlan_qualifier[0x2];
	u8 ip_fragmented[0x1];
	u8 functional_lb[0x1];
};

struct mlx5_ifc_definer_hl_ib_l2_bits {
	u8 sx_sniffer[0x1];
	u8 force_lb[0x1];
	u8 functional_lb[0x1];
	u8 reserved_at_3[0x3];
	u8 port_number[0x2];
	u8 sl[0x4];
	u8 qp_type[0x2];
	u8 lnh[0x2];
	u8 dlid[0x10];
	u8 vl[0x4];
	u8 lrh_packet_length[0xc];
	u8 slid[0x10];
};

struct mlx5_ifc_definer_hl_eth_l3_bits {
	u8 ip_version[0x4];
	u8 ihl[0x4];
	union {
		u8 tos[0x8];
		struct {
			u8 dscp[0x6];
			u8 ecn[0x2];
		};
	};
	u8 time_to_live_hop_limit[0x8];
	u8 protocol_next_header[0x8];
	u8 identification[0x10];
	union {
		u8 ipv4_frag[0x10];
		struct {
			u8 flags[0x3];
			u8 fragment_offset[0xd];
		};
	};
	u8 ipv4_total_length[0x10];
	u8 checksum[0x10];
	u8 reserved_at_60[0xc];
	u8 flow_label[0x14];
	u8 eth_packet_length[0x10];
	u8 ipv6_payload_length[0x10];
};

struct mlx5_ifc_definer_hl_eth_l4_bits {
	u8 source_port[0x10];
	u8 destination_port[0x10];
	u8 data_offset[0x4];
	u8 l4_ok[0x1];
	u8 l3_ok[0x1];
	u8 ip_fragmented[0x1];
	u8 tcp_ns[0x1];
	union {
		u8 tcp_flags[0x8];
		struct {
			u8 tcp_cwr[0x1];
			u8 tcp_ece[0x1];
			u8 tcp_urg[0x1];
			u8 tcp_ack[0x1];
			u8 tcp_psh[0x1];
			u8 tcp_rst[0x1];
			u8 tcp_syn[0x1];
			u8 tcp_fin[0x1];
		};
	};
	u8 first_fragment[0x1];
	u8 reserved_at_31[0xf];
};

struct mlx5_ifc_definer_hl_src_qp_gvmi_bits {
	u8 loopback_syndrome[0x8];
	u8 l3_type[0x2];
	u8 l4_type_bwc[0x2];
	u8 first_vlan_qualifier[0x2];
	u8 reserved_at_e[0x1];
	u8 functional_lb[0x1];
	u8 source_gvmi[0x10];
	u8 force_lb[0x1];
	u8 ip_fragmented[0x1];
	u8 source_is_requestor[0x1];
	u8 reserved_at_23[0x5];
	u8 source_qp[0x18];
};

struct mlx5_ifc_definer_hl_ib_l4_bits {
	u8 opcode[0x8];
	u8 qp[0x18];
	u8 se[0x1];
	u8 migreq[0x1];
	u8 ackreq[0x1];
	u8 fecn[0x1];
	u8 becn[0x1];
	u8 bth[0x1];
	u8 deth[0x1];
	u8 dcceth[0x1];
	u8 reserved_at_28[0x2];
	u8 pad_count[0x2];
	u8 tver[0x4];
	u8 p_key[0x10];
	u8 reserved_at_40[0x8];
	u8 deth_source_qp[0x18];
};

enum integrity_ok1_bits {
	DEFINER_OKS1_FIRST_L4_OK = 24,
	DEFINER_OKS1_FIRST_L3_OK = 25,
	DEFINER_OKS1_SECOND_L4_OK = 26,
	DEFINER_OKS1_SECOND_L3_OK = 27,
	DEFINER_OKS1_FIRST_L4_CSUM_OK = 28,
	DEFINER_OKS1_FIRST_IPV4_CSUM_OK = 29,
	DEFINER_OKS1_SECOND_L4_CSUM_OK = 30,
	DEFINER_OKS1_SECOND_IPV4_CSUM_OK = 31,
};

struct mlx5_ifc_definer_hl_oks1_bits {
	union {
		u8 oks1_bits[0x20];
		struct {
			u8 second_ipv4_checksum_ok[0x1];
			u8 second_l4_checksum_ok[0x1];
			u8 first_ipv4_checksum_ok[0x1];
			u8 first_l4_checksum_ok[0x1];
			u8 second_l3_ok[0x1];
			u8 second_l4_ok[0x1];
			u8 first_l3_ok[0x1];
			u8 first_l4_ok[0x1];
			u8 flex_parser7_steering_ok[0x1];
			u8 flex_parser6_steering_ok[0x1];
			u8 flex_parser5_steering_ok[0x1];
			u8 flex_parser4_steering_ok[0x1];
			u8 flex_parser3_steering_ok[0x1];
			u8 flex_parser2_steering_ok[0x1];
			u8 flex_parser1_steering_ok[0x1];
			u8 flex_parser0_steering_ok[0x1];
			u8 second_ipv6_extension_header_vld[0x1];
			u8 first_ipv6_extension_header_vld[0x1];
			u8 l3_tunneling_ok[0x1];
			u8 l2_tunneling_ok[0x1];
			u8 second_tcp_ok[0x1];
			u8 second_udp_ok[0x1];
			u8 second_ipv4_ok[0x1];
			u8 second_ipv6_ok[0x1];
			u8 second_l2_ok[0x1];
			u8 vxlan_ok[0x1];
			u8 gre_ok[0x1];
			u8 first_tcp_ok[0x1];
			u8 first_udp_ok[0x1];
			u8 first_ipv4_ok[0x1];
			u8 first_ipv6_ok[0x1];
			u8 first_l2_ok[0x1];
		};
	};
};

struct mlx5_ifc_definer_hl_oks2_bits {
	u8 reserved_at_0[0xa];
	u8 second_mpls_ok[0x1];
	u8 second_mpls4_s_bit[0x1];
	u8 second_mpls4_qualifier[0x1];
	u8 second_mpls3_s_bit[0x1];
	u8 second_mpls3_qualifier[0x1];
	u8 second_mpls2_s_bit[0x1];
	u8 second_mpls2_qualifier[0x1];
	u8 second_mpls1_s_bit[0x1];
	u8 second_mpls1_qualifier[0x1];
	u8 second_mpls0_s_bit[0x1];
	u8 second_mpls0_qualifier[0x1];
	u8 first_mpls_ok[0x1];
	u8 first_mpls4_s_bit[0x1];
	u8 first_mpls4_qualifier[0x1];
	u8 first_mpls3_s_bit[0x1];
	u8 first_mpls3_qualifier[0x1];
	u8 first_mpls2_s_bit[0x1];
	u8 first_mpls2_qualifier[0x1];
	u8 first_mpls1_s_bit[0x1];
	u8 first_mpls1_qualifier[0x1];
	u8 first_mpls0_s_bit[0x1];
	u8 first_mpls0_qualifier[0x1];
};

struct mlx5_ifc_definer_hl_voq_bits {
	u8 reserved_at_0[0x18];
	u8 ecn_ok[0x1];
	u8 congestion[0x1];
	u8 profile[0x2];
	u8 internal_prio[0x4];
};

struct mlx5_ifc_definer_hl_ipv4_src_dst_bits {
	u8 source_address[0x20];
	u8 destination_address[0x20];
};

struct mlx5_ifc_definer_hl_random_number_bits {
	u8 random_number[0x10];
	u8 reserved[0x10];
};

struct mlx5_ifc_definer_hl_ipv6_addr_bits {
	u8 ipv6_address_127_96[0x20];
	u8 ipv6_address_95_64[0x20];
	u8 ipv6_address_63_32[0x20];
	u8 ipv6_address_31_0[0x20];
};

struct mlx5_ifc_definer_udp_misc_bits {
	u8 length[0x10];
	u8 checksum[0x10];
};

struct mlx5_ifc_definer_tcp_icmp_header_bits {
	union {
		struct {
			union {
				u8 icmp_dw0[0x20];
				struct {
					u8 icmp_type[0x8];
					u8 icmp_code[0x8];
					u8 icmp_cksum[0x10];
				};
			};
			u8 icmp_dw1[0x20];
			u8 icmp_dw2[0x20];
		};
		struct {
			u8 tcp_seq[0x20];
			u8 tcp_ack[0x20];
			union {
				u8 tcp_win_urg[0x20];
				struct {
					u8 tcp_win[0x10];
					u8 tcp_urg[0x10];
				};
			};
		};
	};
};

union mlx5_ifc_definer_hl_tunnel_header_bits {
	struct {
		u8 tunnel_header_0[0x20];
		u8 tunnel_header_1[0x20];
		u8 tunnel_header_2[0x20];
		u8 tunnel_header_3[0x20];
	};
	struct mlx5_ifc_header_gre_bits gre;
	struct mlx5_ifc_header_nvgre_bits nvgre;
	struct mlx5_ifc_header_vxlan_bits vxlan;
	struct mlx5_ifc_header_vxlan_gpe_bits vxlan_gpe;
	struct mlx5_ifc_header_vxlan_gbp_bits vxlan_gbp;
};

struct mlx5_ifc_definer_hl_ipsec_bits {
	u8 spi[0x20];
	u8 sequence_number[0x20];
	u8 reserved[0x10];
	u8 ipsec_syndrome[0x8];
	u8 next_header[0x8];
};

struct mlx5_ifc_definer_hl_metadata_bits {
	u8 metadata_to_cqe[0x20];
	u8 general_purpose[0x20];
	u8 acomulated_hash[0x20];
};

struct mlx5_ifc_utc_timestamp {
	u8 utc_timestamp_h[0x20];
	u8 utc_timestamp_l[0x20];
};

struct mlx5_ifc_free_running_timestamp {
	u8 frc_timestamp_h[0x20];
	u8 frc_timestamp_l[0x20];
};

struct mlx5_ifc_definer_hl_flex_parser_bits {
	u8 flex_parser_7[0x20];
	u8 flex_parser_6[0x20];
	u8 flex_parser_5[0x20];
	u8 flex_parser_4[0x20];
	u8 flex_parser_3[0x20];
	u8 flex_parser_2[0x20];
	u8 flex_parser_1[0x20];
	u8 flex_parser_0[0x20];
};

struct mlx5_ifc_definer_hl_registers_bits {
	u8 register_c_10[0x20];
	u8 register_c_11[0x20];
	u8 register_c_8[0x20];
	u8 register_c_9[0x20];
	u8 register_c_6[0x20];
	u8 register_c_7[0x20];
	u8 register_c_4[0x20];
	u8 register_c_5[0x20];
	u8 register_c_2[0x20];
	u8 register_c_3[0x20];
	u8 register_c_0[0x20];
	u8 register_c_1[0x20];
};

struct mlx5_ifc_definer_hl_mpls_bits {
	u8 mpls0_label[0x20];
	u8 mpls1_label[0x20];
	u8 mpls2_label[0x20];
	u8 mpls3_label[0x20];
	u8 mpls4_label[0x20];
};

struct mlx5_ifc_definer_hl_psp_bits {
	u8 psp_dw0[0x20];
	u8 psp_dw1[0x20];
	u8 psp_dw2[0x20];
	u8 psp_dw3[0x20];
	u8 psp_dw4[0x20];
	u8 psp_dw5[0x20];
	u8 psp_dw6[0x20];
	u8 psp_dw7[0x20];
	u8 psp_dw8[0x20];
	u8 psp_dw9[0x20];
	u8 reserved[0x18];
	u8 psp_syndrome[0x8];
};

struct mlx5_ifc_definer_hl_bits {
	struct mlx5_ifc_definer_hl_eth_l2_bits eth_l2_outer;
	struct mlx5_ifc_definer_hl_eth_l2_bits eth_l2_inner;
	struct mlx5_ifc_definer_hl_eth_l2_src_bits eth_l2_src_outer;
	struct mlx5_ifc_definer_hl_eth_l2_src_bits eth_l2_src_inner;
	struct mlx5_ifc_definer_hl_ib_l2_bits ib_l2;
	struct mlx5_ifc_definer_hl_eth_l3_bits eth_l3_outer;
	struct mlx5_ifc_definer_hl_eth_l3_bits eth_l3_inner;
	struct mlx5_ifc_definer_hl_eth_l4_bits eth_l4_outer;
	struct mlx5_ifc_definer_hl_eth_l4_bits eth_l4_inner;
	struct mlx5_ifc_definer_hl_src_qp_gvmi_bits source_qp_gvmi;
	struct mlx5_ifc_definer_hl_ib_l4_bits ib_l4;
	struct mlx5_ifc_definer_hl_oks1_bits oks1;
	struct mlx5_ifc_definer_hl_oks2_bits oks2;
	struct mlx5_ifc_definer_hl_voq_bits voq;
	u8 reserved_at_480[0x380];
	struct mlx5_ifc_definer_hl_ipv4_src_dst_bits ipv4_src_dest_outer;
	struct mlx5_ifc_definer_hl_ipv4_src_dst_bits ipv4_src_dest_inner;
	struct mlx5_ifc_definer_hl_ipv6_addr_bits ipv6_dst_outer;
	struct mlx5_ifc_definer_hl_ipv6_addr_bits ipv6_dst_inner;
	struct mlx5_ifc_definer_hl_ipv6_addr_bits ipv6_src_outer;
	struct mlx5_ifc_definer_hl_ipv6_addr_bits ipv6_src_inner;
	u8 unsupported_dest_ib_l3[0x80];
	u8 unsupported_source_ib_l3[0x80];
	struct mlx5_ifc_definer_udp_misc_bits l4_misc_outer;
	struct mlx5_ifc_definer_udp_misc_bits l4_misc_inner;
	struct mlx5_ifc_definer_tcp_icmp_header_bits tcp_icmp;
	union mlx5_ifc_definer_hl_tunnel_header_bits tunnel_header;
	struct mlx5_ifc_definer_hl_mpls_bits mpls_outer;
	struct mlx5_ifc_definer_hl_mpls_bits mpls_inner;
	u8 unsupported_config_headers_outer[0x80];
	u8 unsupported_config_headers_inner[0x80];
	struct mlx5_ifc_definer_hl_random_number_bits random_number;
	struct mlx5_ifc_definer_hl_ipsec_bits ipsec;
	struct mlx5_ifc_definer_hl_metadata_bits metadata;
	struct mlx5_ifc_utc_timestamp utc;
	struct mlx5_ifc_free_running_timestamp frc;
	struct mlx5_ifc_definer_hl_flex_parser_bits flex_parser;
	struct mlx5_ifc_definer_hl_registers_bits registers;
	u8 reserved[0x220];
	struct mlx5_ifc_definer_hl_psp_bits psp;
	/* Reserved in case header layout on future HW */
	u8 unsupported_reserved[0x9c0];
};

struct definer {
	enum definer_type type;
	uint8_t dw_selector[DW_SELECTORS];
	uint8_t byte_selector[BYTE_SELECTORS];
	struct rule_match_tag *mask;
	struct mlx5dv_devx_obj *obj;
};

struct definer_fc {
	uint8_t item_idx;
	uint8_t bit_sz;
	uint32_t bit_off;
	/* Fetch offset from item */
	uint32_t src_bit_off;

	void *user_conv_data;
	void (*user_conv_func)(const struct mlx5dv_hws_item_data *item_data,
			       void *user_conv_data,
			       enum mlx5dv_hws_field_user_conv_flag conv_flag,
			       uint32_t *ret_conv_data);

	struct { // TODO split to improve cache
		enum mlx5dv_hws_field_name fname;
		enum mlx5dv_hws_field_header header;
		enum mlx5dv_hws_field_match_op op;
		enum mlx5dv_hws_field_match_type type;
		uint16_t compare_dw;
		/* Mask size based on largest field, currently 32bit */
		uint32_t mask;
	} ctrl;
};

struct definer_cache {
	struct list_head head;
};

struct definer_cache_item {
	struct definer definer;
	uint32_t refcount;
	struct list_node next;
};

static inline bool
definer_is_jumbo(struct definer *definer)
{
	return (definer->type == DEFINER_TYPE_JUMBO);
}

void definer_create_tag(struct mlx5dv_hws_item_data *data,
			struct definer_fc *fc,
			uint32_t fc_sz,
			uint8_t *tag);

void definer_create_tag_mask(struct definer_fc *fc,
			     uint32_t fc_sz,
			     uint8_t *tag);

void definer_create_tag_range(struct mlx5dv_hws_item_data *data,
			      struct definer_fc *fcr,
			      uint32_t fcr_sz,
			      uint8_t *tag);

void definer_create_tag_compare(struct mlx5dv_hws_item_data *data,
				struct definer_fc *fcc,
				uint32_t fcc_sz,
				uint8_t *tag);

int definer_get_id(struct definer *definer);

int definer_matcher_init(struct mlx5dv_hws_context *ctx,
			 struct mlx5dv_hws_matcher *matcher);

void definer_matcher_uninit(struct mlx5dv_hws_matcher *matcher);

int definer_cache_init(struct definer_cache **cache);

void definer_cache_uninit(struct definer_cache *cache);

int definer_compare(struct definer *definer_a, struct definer *definer_b);

int definer_mt_bind_items(struct mlx5dv_hws_context *ctx,
			  struct mlx5dv_hws_mt *mt,
			  const struct mlx5dv_hws_item items[],
			  uint8_t num_of_items);

int definer_mt_check_root(struct mlx5dv_hws_mt *mt);

int definer_mt_align(struct mlx5dv_hws_context *ctx,
		     struct mlx5dv_hws_mt *mt,
		     bool is_root);

void definer_mt_unbind_items(struct mlx5dv_hws_mt *mt);

enum mlx5dv_hws_field_name
definer_hl_to_fname(enum mlx5dv_hws_field_header hws_header,
		    uint32_t bit_off,
		    uint32_t bit_sz);

uint32_t definer_get_field_width(enum mlx5dv_hws_field_name fname);

void definer_set_dependent_fields_root(struct definer_fc *fc,
				       uint32_t fc_sz,
				       uint8_t *match_param,
				       bool is_mask);

#endif /* _HWS_DEFINER_H_ */
