/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_INTERNAL_H_
#define _HWS_INTERNAL_H_

#include <unistd.h>

#include "../mlx5.h"
#include "../mlx5dv.h"

#define W_SIZE 2
#define DW_SIZE 4
#define QW_SIZE 8
#define BITS_IN_BYTE 8
#define BITS_IN_DW (BITS_IN_BYTE * DW_SIZE)
#define IS_BIT_SET(_value, _bit) ((_value) & BIT(_bit))
#define DW_LOG_SZ 2
#define DW_LOG_BIT_SZ 5

enum ste_sz {
	STE_MATCH_TAG_SZ = 32,
	STE_JUMBO_TAG_SZ = 44,
	STE_ACTION_SZ = 12,
	STE_SZ = 64,
};

static inline bool
is_power_of_2(unsigned int v)
{
	return !(v & (v - 1));
}

#include "mlx5dv_hws.h"
#include "hws_mlx5_ifc.h"
#include "hws_crc.h"
#include "hws_cmd.h"
#include "hws_parser.h"
#include "hws_pool.h"
#include "hws_context.h"
#include "hws_send.h"
#include "hws_resource.h"
#include "hws_table.h"
#include "hws_action.h"
#include "hws_definer.h"
#include "hws_matcher.h"
#include "hws_rule.h"
#include "hws_pat_and_arg.h"
#include "hws_debug.h"

#endif /* _HWS_INTERNAL_H_ */
