/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_MLX5_IFC_H_
#define _HWS_MLX5_IFC_H_

typedef uint64_t u64;
typedef uint32_t u32;
typedef uint16_t u16;
typedef uint8_t u8;

#define MLX5_IFC_MAX_ACTIONS_DATA_IN_HEADER_MODIFY 512
#define MLX5_IFC_RTC_LINEAR_LOOKUP_TBL_LOG_MAX 16
#define MLX5_IFC_MULTI_PATH_FT_MAX_LEVEL 64


/* HCA bit masks indicating which Flex parser protocols are already enabled. */
#define MLX5_HCA_FLEX_IPV4_OVER_VXLAN_ENABLED (1UL << 0)
#define MLX5_HCA_FLEX_IPV6_OVER_VXLAN_ENABLED (1UL << 1)
#define MLX5_HCA_FLEX_IPV6_OVER_IP_ENABLED (1UL << 2)
#define MLX5_HCA_FLEX_GENEVE_ENABLED (1UL << 3)
#define MLX5_HCA_FLEX_CW_MPLS_OVER_GRE_ENABLED (1UL << 4)
#define MLX5_HCA_FLEX_CW_MPLS_OVER_UDP_ENABLED (1UL << 5)
#define MLX5_HCA_FLEX_P_BIT_VXLAN_GPE_ENABLED (1UL << 6)
#define MLX5_HCA_FLEX_VXLAN_GPE_ENABLED (1UL << 7)
#define MLX5_HCA_FLEX_ICMP_ENABLED (1UL << 8)
#define MLX5_HCA_FLEX_ICMPV6_ENABLED (1UL << 9)
#define MLX5_HCA_FLEX_GTPU_ENABLED (1UL << 11)
#define MLX5_HCA_FLEX_GTPU_DW_2_ENABLED (1UL << 16)
#define MLX5_HCA_FLEX_GTPU_FIRST_EXT_DW_0_ENABLED (1UL << 17)
#define MLX5_HCA_FLEX_GTPU_DW_0_ENABLED (1UL << 18)
#define MLX5_HCA_FLEX_GTPU_TEID_ENABLED (1UL << 19)

enum {
	MLX5_CMD_OP_QUERY_HCA_CAP = 0x100,
	MLX5_CMD_OP_CREATE_MKEY = 0x200,
	MLX5_CMD_OP_CREATE_CQ = 0x400,
	MLX5_CMD_OP_CREATE_QP = 0x500,
	MLX5_CMD_OP_RST2INIT_QP = 0x502,
	MLX5_CMD_OP_INIT2RTR_QP = 0x503,
	MLX5_CMD_OP_RTR2RTS_QP = 0x504,
	MLX5_CMD_OP_RTS2RTS_QP = 0x505,
	MLX5_CMD_OP_SQERR2RTS_QP = 0x506,
	MLX5_CMD_OP_QP_2ERR = 0x507,
	MLX5_CMD_OP_QP_2RST = 0x50a,
	MLX5_CMD_OP_QUERY_QP = 0x50b,
	MLX5_CMD_OP_SQD2RTS_QP = 0x50c,
	MLX5_CMD_OP_INIT2INIT_QP = 0x50e,
	MLX5_CMD_OP_QUERY_NIC_VPORT_CONTEXT = 0x754,
	MLX5_CMD_OP_ALLOC_Q_COUNTER = 0x771,
	MLX5_CMD_OP_QUERY_Q_COUNTER = 0x773,
	MLX5_CMD_OP_CREATE_SQ = 0X904,
	MLX5_CMD_OP_MODIFY_SQ = 0X905,
	MLX5_CMD_OP_CREATE_FLOW_TABLE = 0x930,
	MLX5_CMD_OP_QUERY_FLOW_TABLE = 0x932,
	MLX5_CMD_OP_CREATE_FLOW_GROUP = 0x933,
	MLX5_CMD_OP_SET_FLOW_TABLE_ENTRY = 0x936,
	MLX5_CMD_OP_MODIFY_FLOW_TABLE = 0x93c,
	MLX5_CMD_OP_ALLOC_PACKET_REFORMAT_CONTEXT = 0x93d,
	MLX5_CMD_OP_DEALLOC_PACKET_REFORMAT_CONTEXT = 0x93e,
	MLX5_CMD_OP_ALLOC_FLOW_COUNTER = 0x939,
	MLX5_CMD_OP_QUERY_FLOW_COUNTER = 0x93b,
	MLX5_CMD_OP_CREATE_GENERAL_OBJECT = 0xa00,
	MLX5_CMD_OP_MODIFY_GENERAL_OBJECT = 0xa01,
	MLX5_CMD_OP_QUERY_GENERAL_OBJECT = 0xa02,
	MLX5_CMD_OP_QUERY_MATCH_SAMPLE_INFO = 0xb13,
	MLX5_CMD_OP_ALLOW_OTHER_VHCA_ACCESS = 0xb16,
	MLX5_CMD_OP_GENERATE_WQE = 0xb17,
};

enum {
	MLX5_MODIFICATION_TYPE_SET = 0x1,
	MLX5_MODIFICATION_TYPE_ADD = 0x2,
	MLX5_MODIFICATION_TYPE_COPY = 0x3,
	MLX5_MODIFICATION_TYPE_INSERT = 0x4,
	MLX5_MODIFICATION_TYPE_REMOVE = 0x5,
	MLX5_MODIFICATION_TYPE_NOP = 0x6,
	MLX5_MODIFICATION_TYPE_REMOVE_WORDS = 0x7,
	MLX5_MODIFICATION_TYPE_ADD_FIELD = 0x8,
	MLX5_MODIFICATION_TYPE_MAX,
};

/* The field of packet to be modified. */
enum mlx5_modification_field {
	MLX5_MODI_OUT_NONE = -1,
	MLX5_MODI_OUT_SMAC_47_16 = 1,
	MLX5_MODI_OUT_SMAC_15_0,
	MLX5_MODI_OUT_ETHERTYPE,
	MLX5_MODI_OUT_DMAC_47_16,
	MLX5_MODI_OUT_DMAC_15_0,
	MLX5_MODI_OUT_IP_DSCP,
	MLX5_MODI_OUT_TCP_FLAGS,
	MLX5_MODI_OUT_TCP_SPORT,
	MLX5_MODI_OUT_TCP_DPORT,
	MLX5_MODI_OUT_IPV4_TTL,
	MLX5_MODI_OUT_UDP_SPORT,
	MLX5_MODI_OUT_UDP_DPORT,
	MLX5_MODI_OUT_SIPV6_127_96,
	MLX5_MODI_OUT_SIPV6_95_64,
	MLX5_MODI_OUT_SIPV6_63_32,
	MLX5_MODI_OUT_SIPV6_31_0,
	MLX5_MODI_OUT_DIPV6_127_96,
	MLX5_MODI_OUT_DIPV6_95_64,
	MLX5_MODI_OUT_DIPV6_63_32,
	MLX5_MODI_OUT_DIPV6_31_0,
	MLX5_MODI_OUT_SIPV4,
	MLX5_MODI_OUT_DIPV4,
	MLX5_MODI_OUT_FIRST_VID,
	MLX5_MODI_IN_SMAC_47_16 = 0x31,
	MLX5_MODI_IN_SMAC_15_0,
	MLX5_MODI_IN_ETHERTYPE,
	MLX5_MODI_IN_DMAC_47_16,
	MLX5_MODI_IN_DMAC_15_0,
	MLX5_MODI_IN_IP_DSCP,
	MLX5_MODI_IN_TCP_FLAGS,
	MLX5_MODI_IN_TCP_SPORT,
	MLX5_MODI_IN_TCP_DPORT,
	MLX5_MODI_IN_IPV4_TTL,
	MLX5_MODI_IN_UDP_SPORT,
	MLX5_MODI_IN_UDP_DPORT,
	MLX5_MODI_IN_SIPV6_127_96,
	MLX5_MODI_IN_SIPV6_95_64,
	MLX5_MODI_IN_SIPV6_63_32,
	MLX5_MODI_IN_SIPV6_31_0,
	MLX5_MODI_IN_DIPV6_127_96,
	MLX5_MODI_IN_DIPV6_95_64,
	MLX5_MODI_IN_DIPV6_63_32,
	MLX5_MODI_IN_DIPV6_31_0,
	MLX5_MODI_IN_SIPV4,
	MLX5_MODI_IN_DIPV4,
	MLX5_MODI_OUT_IPV6_HOPLIMIT,
	MLX5_MODI_IN_IPV6_HOPLIMIT,
	MLX5_MODI_META_DATA_REG_A,
	MLX5_MODI_OUT_IP_PROTOCOL,
	MLX5_MODI_META_DATA_REG_B = 0x50,
	MLX5_MODI_META_REG_C_0,
	MLX5_MODI_META_REG_C_1,
	MLX5_MODI_META_REG_C_2,
	MLX5_MODI_META_REG_C_3,
	MLX5_MODI_META_REG_C_4,
	MLX5_MODI_META_REG_C_5,
	MLX5_MODI_META_REG_C_6,
	MLX5_MODI_META_REG_C_7,
	MLX5_MODI_OUT_TCP_SEQ_NUM,
	MLX5_MODI_IN_TCP_SEQ_NUM,
	MLX5_MODI_OUT_TCP_ACK_NUM,
	MLX5_MODI_IN_TCP_ACK_NUM,
	MLX5_MODI_OUT_ESP_SPI = 0x5E,
	MLX5_MODI_IN_ESP_SPI,
	MLX5_MODI_GTP_TEID = 0x6E,
	MLX5_MODI_OUT_IP_ECN = 0x73,
	MLX5_MODI_IN_IP_ECN,
	MLX5_MODI_TUNNEL_HDR_DW_1,
	MLX5_MODI_GTPU_FIRST_EXT_DW_0,
	MLX5_MODI_PSP_HEADER_0,
	MLX5_MODI_PSP_HEADER_1,
	MLX5_MODI_PSP_HEADER_2,
	MLX5_MODI_PSP_HEADER_3,
	MLX5_MODI_PSP_HEADER_4,
	MLX5_MODI_PSP_HEADER_5,
	MLX5_MODI_PSP_HEADER_6,
	MLX5_MODI_PSP_HEADER_7,
	MLX5_MODI_PSP_HEADER_8,
	MLX5_MODI_PSP_HEADER_9,
	MLX5_MODI_HASH_RESULT,
	MLX5_MODI_OUT_ESP_SEQ_NUM,
	MLX5_MODI_IN_ESP_SEQ_NUM,
	MLX5_MODI_IN_MPLS_LABEL_0 = 0x8a,
	MLX5_MODI_IN_MPLS_LABEL_1,
	MLX5_MODI_IN_MPLS_LABEL_2,
	MLX5_MODI_IN_MPLS_LABEL_3,
	MLX5_MODI_IN_MPLS_LABEL_4,
	MLX5_MODI_META_REG_C_8,
	MLX5_MODI_META_REG_C_9,
	MLX5_MODI_META_REG_C_10,
	MLX5_MODI_META_REG_C_11,
	MLX5_MODI_META_REG_C_12,
	MLX5_MODI_META_REG_C_13,
	MLX5_MODI_META_REG_C_14,
	MLX5_MODI_META_REG_C_15,
	MLX5_MODI_OUT_IPV6_TRAFFIC_CLASS = 0x11C,
	MLX5_MODI_OUT_IPV4_TOTAL_LEN,
	MLX5_MODI_OUT_IPV6_PAYLOAD_LEN,
	MLX5_MODI_OUT_IPV4_IHL,
	MLX5_MODI_OUT_TCP_DATA_OFFSET,
	MLX5_MODI_IN_IPV6_TRAFFIC_CLASS,
	MLX5_MODI_IN_IPV4_TOTAL_LEN,
	MLX5_MODI_IN_IPV6_PAYLOAD_LEN,
	MLX5_MODI_IN_IPV4_IHL,
	MLX5_MODI_IN_TCP_DATA_OFFSET,
	MLX5_MODI_OUT_IPSEC_NEXT_HDR,
	MLX5_MODI_OUT_IPV6_FLOW_LABEL,
	MLX5_MODI_IN_IPV6_FLOW_LABEL,
	//MLX5_MODI_INVALID = INT_MAX,
};

enum mlx5_ifc_rtc_update_mode {
	MLX5_IFC_RTC_STE_UPDATE_MODE_BY_HASH = 0x0,
	MLX5_IFC_RTC_STE_UPDATE_MODE_BY_OFFSET = 0x1,
};

enum mlx5_ifc_rtc_access_mode {
	MLX5_IFC_RTC_STE_ACCESS_MODE_BY_HASH = 0x0,
	MLX5_IFC_RTC_STE_ACCESS_MODE_LINEAR = 0x1,
};

enum mlx5_ifc_rtc_ste_format {
	MLX5_IFC_RTC_STE_FORMAT_8DW = 0x4,
	MLX5_IFC_RTC_STE_FORMAT_11DW = 0x5,
	MLX5_IFC_RTC_STE_FORMAT_RANGE = 0x7,
	MLX5_IFC_RTC_STE_FORMAT_4DW_RANGE = 0x8,
};

enum mlx5_ifc_rtc_reparse_mode {
	MLX5_IFC_RTC_REPARSE_NEVER = 0x0,
	MLX5_IFC_RTC_REPARSE_ALWAYS = 0x1,
	MLX5_IFC_RTC_REPARSE_BY_STC = 0x2,
};

struct mlx5_ifc_rtc_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x20];
	u8 other_vport[0x1];
	u8 reserved_at_61[0xf];
	u8 vport_number[0x10];
	u8 update_index_mode[0x2];
	u8 reparse_mode[0x2];
	u8 num_match_ste[0x4];
	u8 pd[0x18];
	u8 reserved_at_a0[0x9];
	u8 access_index_mode[0x3];
	u8 num_hash_definer[0x4];
	u8 update_method[0x1];
	u8 reserved_at_b1[0x2];
	u8 log_depth[0x5];
	u8 log_hash_size[0x8];
	u8 ste_format_0[0x8];
	u8 table_type[0x8];
	u8 ste_format_1[0x8];
	u8 reserved_at_d8[0x8];
	u8 match_definer_0[0x20];
	u8 stc_id[0x20];
	u8 ste_table_base_id[0x20];
	u8 ste_table_offset[0x20];
	u8 reserved_at_160[0x8];
	u8 miss_flow_table_id[0x18];
	u8 match_definer_1[0x20];
	u8 reserved_at_1a0[0x260];
};

enum mlx5_ifc_ste_match_4dw_range_match_op {
	MLX5_IFC_STE_MATCH_4DW_RANGE_MATCH_OP_BIGGER = 0,
	MLX5_IFC_STE_MATCH_4DW_RANGE_MATCH_OP_SMALLER = 1,
	MLX5_IFC_STE_MATCH_4DW_RANGE_MATCH_OP_BETWEEN = 2,
};

struct mlx5_ifc_ste_match_4dw_range_ctrl_dw_bits {
	u8 actions[0x60];
	u8 num_cond[0x1];
	u8 reserved_at_1[0x2];
	u8 cond1_base_src[0x1];
	u8 cond1_inverse[0x1];
	u8 reserved_at_5[0x1];
	u8 cond1_operator[0x2];
	u8 reserved_at_8[0x3];
	u8 cond0_base_src[0x1];
	u8 cond0_inverse[0x1];
	u8 reserved_at_a[0x1];
	u8 cond0_operator[0x2];
	u8 compare_delta_47_32[0x10];
	u8 compare_delta_31_0[0x20];
	u8 match_base_1[0x20];
	u8 match_base_0[0x20];
	u8 match_value[0x80];
};

enum {
	MLX5_GENERAL_OBJ_TYPE_GENEVE_TLV_OPT = 0x000b,
	MLX5_GENERAL_OBJ_TYPE_DEK = 0x000c,
	MLX5_GENERAL_OBJ_TYPE_VIRTQ = 0x000d,
	MLX5_GENERAL_OBJ_TYPE_DEFINER = 0x0018,
	MLX5_GENERAL_OBJ_TYPE_VIRTIO_Q_COUNTERS = 0x001c,
	MLX5_GENERAL_OBJ_TYPE_IMPORT_KEK = 0x001d,
	MLX5_GENERAL_OBJ_TYPE_CREDENTIAL = 0x001e,
	MLX5_GENERAL_OBJ_TYPE_CRYPTO_LOGIN = 0x001f,
	MLX5_GENERAL_OBJ_TYPE_FLEX_PARSE_GRAPH = 0x0022,
	MLX5_GENERAL_OBJ_TYPE_FLOW_METER_ASO = 0x0024,
	MLX5_GENERAL_OBJ_TYPE_FLOW_HIT_ASO = 0x0025,
	MLX5_GENERAL_OBJ_TYPE_CONN_TRACK_OFFLOAD = 0x0031,
	MLX5_GENERAL_OBJ_TYPE_ARG = 0x0023,
	MLX5_GENERAL_OBJ_TYPE_STC = 0x0040,
	MLX5_GENERAL_OBJ_TYPE_RTC = 0x0041,
	MLX5_GENERAL_OBJ_TYPE_STE = 0x0042,
	MLX5_GENERAL_OBJ_TYPE_MODIFY_HEADER_PATTERN = 0x0043,
	MLX5_GENERAL_OBJ_TYPE_FT_ALIAS = 0xff15,
	MLX5_GENERAL_OBJ_TYPE_TIR_ALIAS = 0xff16,
};

struct mlx5_ifc_general_obj_in_cmd_hdr_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x20];
	u8 obj_type[0x10];
	u8 obj_id[0x20];
	union {
		struct {
			u8 alias_object[0x1];
			u8 reserved_at_61[0x2];
			u8 log_obj_range[0x5];
			u8 reserved_at_68[0x18];
		};
		u8 obj_offset[0x20];
	};
};

struct mlx5_ifc_general_obj_out_cmd_hdr_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 obj_id[0x20];
	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_generate_wqe_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mode[0x10];
	u8 reserved_at_40[0x40];
	u8 reserved_at_80[0x8];
	u8 pdn[0x18];
	u8 reserved_at_a0[0x160];
	u8 wqe_ctrl[0x80];
	u8 wqe_gta_ctrl[0x180];
	u8 wqe_gta_data_0[0x200];
	u8 wqe_gta_data_1[0x200];
};

struct mlx5_ifc_generate_wqe_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x1c0];
	u8 cqe_data[0x200];
};

#define MLX5_ACCESS_KEY_LEN 32

struct mlx5_ifc_allow_other_vhca_access_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 reserved_at_40[0x50];
	u8 object_type_to_be_accessed[0x10];
	u8 object_id_to_be_accessed[0x20];
	u8 reserved_at_c0[0x40];
	union {
		u8 access_key_raw[0x100];
		u8 access_key[8][0x20];
	};
};

struct mlx5_ifc_allow_other_vhca_access_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x40];
};

enum mlx5_ifc_stc_action_type {
	MLX5_IFC_STC_ACTION_TYPE_NOP = 0x00,
	MLX5_IFC_STC_ACTION_TYPE_COPY = 0x05,
	MLX5_IFC_STC_ACTION_TYPE_SET = 0x06,
	MLX5_IFC_STC_ACTION_TYPE_ADD = 0x07,
	MLX5_IFC_STC_ACTION_TYPE_REMOVE_WORDS = 0x08,
	MLX5_IFC_STC_ACTION_TYPE_HEADER_REMOVE = 0x09,
	MLX5_IFC_STC_ACTION_TYPE_HEADER_INSERT = 0x0b,
	MLX5_IFC_STC_ACTION_TYPE_TAG = 0x0c,
	MLX5_IFC_STC_ACTION_TYPE_ACC_MODIFY_LIST = 0x0e,
	MLX5_IFC_STC_ACTION_TYPE_CRYPTO_IPSEC_ENCRYPTION = 0x10,
	MLX5_IFC_STC_ACTION_TYPE_CRYPTO_IPSEC_DECRYPTION = 0x11,
	MLX5_IFC_STC_ACTION_TYPE_ASO = 0x12,
	MLX5_IFC_STC_ACTION_TYPE_TRAILER = 0x13,
	MLX5_IFC_STC_ACTION_TYPE_COUNTER = 0x14,
	MLX5_IFC_STC_ACTION_TYPE_ADD_FIELD = 0x1b,
	MLX5_IFC_STC_ACTION_TYPE_CRYPTO_PSP_ENCRYPTION = 0x1f,
	MLX5_IFC_STC_ACTION_TYPE_CRYPTO_PSP_DECRYPTION = 0x20,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_STE_TABLE = 0x80,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_TIR = 0x81,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_FT = 0x82,
	MLX5_IFC_STC_ACTION_TYPE_DROP = 0x83,
	MLX5_IFC_STC_ACTION_TYPE_ALLOW = 0x84,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_VPORT = 0x85,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_UPLINK = 0x86,
	MLX5_IFC_STC_ACTION_TYPE_JUMP_TO_FT_FDB_RX = 0x87,
};

enum mlx5_ifc_stc_reparse_mode {
	MLX5_IFC_STC_REPARSE_IGNORE = 0x0,
	MLX5_IFC_STC_REPARSE_NEVER = 0x1,
	MLX5_IFC_STC_REPARSE_ALWAYS = 0x2,
};

struct mlx5_ifc_stc_ste_param_ste_table_bits {
	u8 ste_obj_id[0x20];
	u8 match_definer_id[0x20];
	u8 reserved_at_40[0x3];
	u8 log_hash_size[0x5];
	u8 reserved_at_48[0x38];
};

struct mlx5_ifc_stc_ste_param_tir_bits {
	u8 reserved_at_0[0x8];
	u8 tirn[0x18];
	u8 reserved_at_20[0x60];
};

struct mlx5_ifc_stc_ste_param_table_bits {
	u8 reserved_at_0[0x8];
	u8 table_id[0x18];
	u8 reserved_at_20[0x60];
};

struct mlx5_ifc_stc_ste_param_flow_counter_bits {
	u8 flow_counter_id[0x20];
};

enum {
	MLX5_ASO_CT_NUM_PER_OBJ = 1,
	MLX5_ASO_METER_NUM_PER_OBJ = 2,
	MLX5_ASO_IPSEC_NUM_PER_OBJ = 1,
	MLX5_ASO_FIRST_HIT_NUM_PER_OBJ = 512,
	MLX5_ASO_FLOW_COUNTER_NUM_PER_OBJ = 4,
};

enum {
	MLX5_ASO_METER_LOG_NUM_PER_OBJ = 1,
	MLX5_ASO_FIRST_HIT_LOG_NUM_PER_OBJ = 9,
	MLX5_ASO_FLOW_COUNTER_LOG_NUM_PER_OBJ = 2,
};

struct mlx5_ifc_stc_ste_param_execute_aso_bits {
	u8 aso_object_id[0x20];
	u8 return_reg_id[0x4];
	u8 aso_type[0x4];
	u8 reserved_at_28[0x18];
};

struct mlx5_ifc_stc_ste_param_ipsec_encrypt_bits {
	u8 ipsec_object_id[0x20];
};

struct mlx5_ifc_stc_ste_param_ipsec_decrypt_bits {
	u8 ipsec_object_id[0x20];
};

struct mlx5_ifc_stc_ste_param_trailer_bits {
	u8 reserved_at_0[0x8];
	u8 command[0x4];
	u8 reserved_at_c[0x2];
	u8 type[0x2];
	u8 reserved_at_10[0xa];
	u8 length[0x6];
};

struct mlx5_ifc_stc_ste_param_header_modify_list_bits {
	u8 header_modify_pattern_id[0x20];
	u8 header_modify_argument_id[0x20];
};

enum mlx5_ifc_header_anchors {
	MLX5_HEADER_ANCHOR_PACKET_START = 0x0,
	MLX5_HEADER_ANCHOR_MAC = 0x1,
	MLX5_HEADER_ANCHOR_FIRST_VLAN_START = 0x2,
	MLX5_HEADER_ANCHOR_IPV6_IPV4 = 0x07,
	MLX5_HEADER_ANCHOR_ESP = 0x08,
	MLX5_HEADER_ANCHOR_TCP_UDP = 0x09,
	MLX5_HEADER_ANCHOR_TUNNEL_HEADER = 0x0a,
	MLX5_HEADER_ANCHOR_INNER_MAC = 0x13,
	MLX5_HEADER_ANCHOR_INNER_IPV6_IPV4 = 0x19,
	MLX5_HEADER_ANCHOR_INNER_TCP_UDP = 0x1a,
	MLX5_HEADER_ANCHOR_L4_PAYLOAD = 0x1b,
	MLX5_HEADER_ANCHOR_INNER_L4_PAYLOAD = 0x1c,
	MLX5_HEADER_ANCHOR_PSP = 0x1e,
	MLX5_HEADER_ANCHOR_PSP_PAYLOAD = 0x1f,
};

enum mlx5_ifc_aso_opc_mod {
	MLX5_IFC_ASO_OPC_MOD_IPSEC = 0x0,
	MLX5_IFC_ASO_OPC_MOD_CT = 0x1,
	MLX5_IFC_ASO_OPC_MOD_POLICER = 0x2,
	MLX5_IFC_ASO_OPC_MOD_RACE_AVOIDANCE = 0x3,
	MLX5_IFC_ASO_OPC_MOD_FLOW_HIT = 0x4,
};

struct mlx5_ifc_stc_ste_param_remove_bits {
	u8 action_type[0x4];
	u8 decap[0x1];
	u8 reserved_at_5[0x5];
	u8 remove_start_anchor[0x6];
	u8 reserved_at_10[0x2];
	u8 remove_end_anchor[0x6];
	u8 reserved_at_18[0x8];
};

struct mlx5_ifc_stc_ste_param_remove_words_bits {
	u8 action_type[0x4];
	u8 reserved_at_4[0x6];
	u8 remove_start_anchor[0x6];
	u8 reserved_at_10[0x1];
	u8 remove_offset[0x7];
	u8 reserved_at_18[0x2];
	u8 remove_size[0x6];
};

struct mlx5_ifc_stc_ste_param_insert_bits {
	u8 action_type[0x4];
	u8 encap[0x1];
	u8 inline_data[0x1];
	u8 push_esp[0x1];
	u8 reserved_at_7[0x3];
	u8 insert_anchor[0x6];
	u8 reserved_at_10[0x1];
	u8 insert_offset[0x7];
	u8 reserved_at_18[0x1];
	u8 insert_size[0x7];
	u8 insert_argument[0x20];
};

struct mlx5_ifc_set_action_in_bits {
	u8 action_type[0x4];
	u8 field[0xc];
	u8 reserved_at_10[0x3];
	u8 offset[0x5];
	u8 reserved_at_18[0x3];
	u8 length[0x5];
	u8 data[0x20];
};

struct mlx5_ifc_add_action_in_bits {
	u8 action_type[0x4];
	u8 field[0xc];
	u8 reserved_at_10[0x3];
	u8 offset[0x5];
	u8 reserved_at_18[0x3];
	u8 length[0x5];
	u8 data[0x20];
};

struct mlx5_ifc_copy_action_in_bits {
	u8 action_type[0x4];
	u8 src_field[0xc];
	u8 reserved_at_10[0x3];
	u8 src_offset[0x5];
	u8 reserved_at_18[0x3];
	u8 length[0x5];
	u8 reserved_at_20[0x4];
	u8 dst_field[0xc];
	u8 reserved_at_30[0x3];
	u8 dst_offset[0x5];
	u8 reserved_at_38[0x8];
};

struct mlx5_ifc_add_field_in_bits {
	u8 action_type[0x4];
	u8 src_field[0xc];
	u8 reserved_at_10[0x3];
	u8 src_offset[0x5];
	u8 reserved_at_18[0x2];
	u8 length[0x6];
	u8 reserved_at_20[0x4];
	u8 dst_field[0xc];
	u8 reserved_at_30[0x3];
	u8 dst_left_shift[0x5];
	u8 reserved_at_38[0x8];
};

struct mlx5_ifc_stc_ste_param_vport_bits {
	u8 eswitch_owner_vhca_id[0x10];
	u8 vport_number[0x10];
	u8 eswitch_owner_vhca_id_valid[0x1];
	u8 reserved_at_21[0x5f];
};

union mlx5_ifc_stc_param_bits {
	struct mlx5_ifc_stc_ste_param_ste_table_bits ste_table;
	struct mlx5_ifc_stc_ste_param_tir_bits tir;
	struct mlx5_ifc_stc_ste_param_table_bits table;
	struct mlx5_ifc_stc_ste_param_flow_counter_bits counter;
	struct mlx5_ifc_stc_ste_param_header_modify_list_bits modify_header;
	struct mlx5_ifc_stc_ste_param_execute_aso_bits aso;
	struct mlx5_ifc_stc_ste_param_remove_bits remove_header;
	struct mlx5_ifc_stc_ste_param_insert_bits insert_header;
	struct mlx5_ifc_set_action_in_bits add;
	struct mlx5_ifc_set_action_in_bits set;
	struct mlx5_ifc_copy_action_in_bits copy;
	struct mlx5_ifc_stc_ste_param_vport_bits vport;
	struct mlx5_ifc_stc_ste_param_ipsec_encrypt_bits ipsec_encrypt;
	struct mlx5_ifc_stc_ste_param_ipsec_decrypt_bits ipsec_decrypt;
	struct mlx5_ifc_stc_ste_param_trailer_bits trailer;
	u8 reserved_at_0[0x80];
};

enum {
	MLX5_IFC_MODIFY_STC_FIELD_SELECT_NEW_STC = 1 << 0,
};

struct mlx5_ifc_stc_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x20];
	u8 other_vport[0x1];
	u8 reserved_at_61[0xf];
	u8 vport_number[0x10];
	u8 reserved_at_80[0x6];
	u8 reparse_mode[0x2];
	u8 table_type[0x8];
	u8 ste_action_offset[0x8];
	u8 action_type[0x8];
	u8 reserved_at_a0[0x60];
	union mlx5_ifc_stc_param_bits stc_param;
	u8 reserved_at_180[0x280];
};

struct mlx5_ifc_ste_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x20];
	u8 other_vport[0x1];
	u8 reserved_at_61[0xf];
	u8 vport_number[0x10];
	u8 reserved_at_80[0x8];
	u8 table_type[0x8];
	u8 reserved_at_90[0x370];
};

enum {
	MLX5_IFC_DEFINER_FORMAT_ID_SELECT = 61,
};

struct mlx5_ifc_definer_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x50];
	u8 format_id[0x10];
	u8 reserved_at_60[0x60];
	u8 format_select_dw3[0x8];
	u8 format_select_dw2[0x8];
	u8 format_select_dw1[0x8];
	u8 format_select_dw0[0x8];
	u8 format_select_dw7[0x8];
	u8 format_select_dw6[0x8];
	u8 format_select_dw5[0x8];
	u8 format_select_dw4[0x8];
	u8 reserved_at_100[0x18];
	u8 format_select_dw8[0x8];
	u8 reserved_at_120[0x20];
	u8 format_select_byte3[0x8];
	u8 format_select_byte2[0x8];
	u8 format_select_byte1[0x8];
	u8 format_select_byte0[0x8];
	u8 format_select_byte7[0x8];
	u8 format_select_byte6[0x8];
	u8 format_select_byte5[0x8];
	u8 format_select_byte4[0x8];
	u8 reserved_at_180[0x40];
	u8 ctrl[0xa0];
	u8 match_mask[0x160];
};

struct mlx5_ifc_arg_bits {
	u8 rsvd0[0x88];
	u8 access_pd[0x18];
};

struct mlx5_ifc_header_modify_pattern_in_bits {
	u8 modify_field_select[0x40];

	u8 reserved_at_40[0x40];

	u8 pattern_length[0x8];
	u8 reserved_at_88[0x18];

	u8 reserved_at_a0[0x60];

	u8 pattern_data[MLX5_IFC_MAX_ACTIONS_DATA_IN_HEADER_MODIFY * 8];
};

struct mlx5_ifc_alias_context_bits {
	u8 vhca_id_to_be_accessed[0x10];
	u8 reserved_at_10[0xd];
	u8 status[0x3];
	u8 object_id_to_be_accessed[0x20];
	u8 reserved_at_40[0x40];
	union {
		u8 access_key_raw[0x100];
		u8 access_key[8][0x20];
	};
	u8 metadata[0x80];
};

struct mlx5_ifc_create_rtc_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_rtc_bits rtc;
};

struct mlx5_ifc_create_stc_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_stc_bits stc;
};

struct mlx5_ifc_create_ste_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_ste_bits ste;
};

struct mlx5_ifc_create_definer_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_definer_bits definer;
};

enum mlx5_ifc_arg_sz {
	MLX5_IFC_ARG_SZ_ACTION = 8,
	MLX5_IFC_ARG_SZ_DATA = 64,
};

struct mlx5_ifc_create_arg_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_arg_bits arg;
};

struct mlx5_ifc_create_header_modify_pattern_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_header_modify_pattern_in_bits pattern;
};

struct mlx5_ifc_create_alias_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_alias_context_bits alias_ctx;
};

struct mlx5_ifc_alloc_flow_counter_in_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 reserved_at_40[0x8];
	u8 pd[0x18];
	u8 reserved_at_60[0x13];
	u8 log_obj_size[0x5];
	u8 flow_counter_bulk[0x8];
};

struct mlx5_ifc_alloc_flow_counter_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 flow_counter_id[0x20];
	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_tcp_window_params_bits {
	u8 max_ack[0x20];
	u8 max_win[0x20];
	u8 reply_end[0x20];
	u8 sent_end[0x20];
};

struct mlx5_ifc_conn_track_aso_bits {
	struct mlx5_ifc_tcp_window_params_bits reply_dir; /* End of DW3. */
	struct mlx5_ifc_tcp_window_params_bits original_dir; /* End of DW7. */
	u8 last_end[0x20]; /* End of DW8. */
	u8 last_ack[0x20]; /* End of DW9. */
	u8 last_seq[0x20]; /* End of DW10. */
	u8 last_win[0x10];
	u8 reserved_at_170[0xa];
	u8 last_dir[0x1];
	u8 last_index[0x5]; /* End of DW11. */
	u8 reserved_at_180[0x40]; /* End of DW13. */
	u8 reply_direction_tcp_scale[0x4];
	u8 reply_direction_tcp_close_initiated[0x1];
	u8 reply_direction_tcp_liberal_enabled[0x1];
	u8 reply_direction_tcp_data_unacked[0x1];
	u8 reply_direction_tcp_max_ack[0x1];
	u8 reserved_at_1c8[0x8];
	u8 original_direction_tcp_scale[0x4];
	u8 original_direction_tcp_close_initiated[0x1];
	u8 original_direction_tcp_liberal_enabled[0x1];
	u8 original_direction_tcp_data_unacked[0x1];
	u8 original_direction_tcp_max_ack[0x1];
	u8 reserved_at_1d8[0x8]; /* End of DW14. */
	u8 valid[0x1];
	u8 state[0x3];
	u8 freeze_track[0x1];
	u8 reserved_at_1e5[0xb];
	u8 reserved_at_1f0[0x1];
	u8 connection_assured[0x1];
	u8 sack_permitted[0x1];
	u8 challenged_acked[0x1];
	u8 heartbeat[0x1];
	u8 max_ack_window[0x3];
	u8 reserved_at_1f8[0x1];
	u8 retransmission_counter[0x3];
	u8 retranmission_limit_exceeded[0x1];
	u8 retranmission_limit[0x3]; /* End of DW15. */
};

struct mlx5_ifc_conn_track_offload_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x40];
	u8 reserved_at_80[0x8];
	u8 conn_track_aso_access_pd[0x18];
	u8 reserved_at_a0[0x160];
	struct mlx5_ifc_conn_track_aso_bits conn_track_aso;
};

struct mlx5_ifc_flow_hit_aso_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x48];
	u8 access_pd[0x18];
	u8 reserved_at_a0[0x160];
	u8 flag[0x200];
};

struct mlx5_ifc_flow_meter_aso_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x48];
	u8 access_pd[0x18];
	u8 reserved_at_a0[0x160];
	u8 parameters[0x200];
};

struct mlx5_ifc_create_conn_track_aso_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_conn_track_offload_bits conn_track_offload;
};

struct mlx5_ifc_create_flow_hit_aso_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_flow_hit_aso_bits flow_hit_aso;
};

struct mlx5_ifc_create_flow_meter_aso_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_flow_meter_aso_bits flow_meter_aso;
};

enum mlx5_flow_destination_type {
	MLX5_FLOW_DESTINATION_TYPE_VPORT = 0x0,
	MLX5_FLOW_DESTINATION_TYPE_FT = 0x1,
	MLX5_FLOW_DESTINATION_TYPE_TIR = 0x2,
	MLX5_FLOW_DESTINATION_TYPE_NOP = 0xb,
};

struct mlx5_ifc_dest_format_bits {
	u8 destination_type[0x8];
	u8 destination_id[0x18];
	u8 destination_eswitch_owner_vhca_id_valid[0x1];
	u8 packet_reformat[0x1];
	u8 reserved_at_22[0xe];
	u8 destination_eswitch_owner_vhca_id[0x10];
};

struct mlx5_ifc_flow_counter_list_bits {
	u8 flow_counter_id[0x20];
	u8 reserved_at_20[0x20];
};

union mlx5_ifc_dest_format_flow_counter_list_auto_bits {
	struct mlx5_ifc_dest_format_bits dest_format;
	struct mlx5_ifc_flow_counter_list_bits flow_counter_list;
	u8 reserved_at_0[0x40];
};

struct mlx5_ifc_flow_context_bits {
	u8 reserved_at_00[0x20];
	u8 group_id[0x20];
	u8 reserved_at_40[0x8];
	u8 flow_tag[0x18];
	u8 reserved_at_60[0x10];
	u8 action[0x10];
	u8 extended_destination[0x1];
	u8 uplink_hairpin_en[0x1];
	u8 flow_source[0x2];
	u8 encrypt_decrypt_type[0x4];
	u8 destination_list_size[0x18];
	u8 reserved_at_a0[0x8];
	u8 flow_counter_list_size[0x18];
	u8 packet_reformat_id[0x20];
	u8 reserved_at_e0[0x40];
	u8 encrypt_decrypt_obj_id[0x20];
	u8 reserved_at_140[0x16c0];
	union mlx5_ifc_dest_format_flow_counter_list_auto_bits destination[0];

};

struct mlx5_ifc_flow_table_context_bits {
	u8 reformat_en[0x1];
	u8 decap_en[0x1];
	u8 sw_owner[0x1];
	u8 termination_table[0x1];
	u8 table_miss_action[0x4];
	u8 level[0x8];
	u8 rtc_valid[0x1];
	u8 reserved_at_11[0x7];
	u8 log_size[0x8];

	u8 reserved_at_20[0x8];
	u8 table_miss_id[0x18];

	u8 reserved_at_40[0x8];
	u8 lag_master_next_table_id[0x18];

	u8 reserved_at_60[0x60];

	union {
		struct {
			u8 rtc_id_0[0x20];
			u8 rtc_id_1[0x20];
			u8 reserved_at_100[0x40];
		};
		struct {
			u8 sw_owner_icm_root_1[0x40];
			u8 sw_owner_icm_root_0[0x40];
		};
	};
};

enum mlx5_ifc_ft_type {
	MLX5_IFC_FT_NIC_RX = 0x0,
	MLX5_IFC_FT_NIC_TX = 0x1,
	MLX5_IFC_FT_FDB = 0x4,
	MLX5_IFC_FT_FDB_RX = 0xa,
	MLX5_IFC_FT_FDB_TX = 0xb,
	MLX5_IFC_FT_FDB_UNIFIED = 0xc,
	MLX5_IFC_FT_RDMA_TRAN_RX = 0xd,
	MLX5_IFC_FT_RDMA_TRAN_TX = 0xe,
};

struct mlx5_ifc_create_flow_table_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 other_vport[0x1];
	u8 reserved_at_41[0xf];
	u8 vport_number[0x10];

	u8 reserved_at_60[0x20];

	u8 table_type[0x8];
	u8 reserved_at_88[0x18];

	u8 reserved_at_a0[0x20];

	struct mlx5_ifc_flow_table_context_bits flow_table_context;
};

struct mlx5_ifc_create_flow_table_out_bits {
	u8 status[0x8];
	u8 icm_address_63_40[0x18];
	u8 syndrome[0x20];
	u8 icm_address_39_32[0x8];
	u8 table_id[0x18];
	u8 icm_address_31_0[0x20];
};

struct mlx5_ifc_modify_flow_table_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 other_vport[0x1];
	u8 reserved_at_41[0xf];
	u8 vport_number[0x10];

	u8 reserved_at_60[0x10];
	u8 modify_field_select[0x10];

	u8 table_type[0x8];
	u8 reserved_at_88[0x18];

	u8 reserved_at_a0[0x8];
	u8 table_id[0x18];

	struct mlx5_ifc_flow_table_context_bits flow_table_context;
};

struct mlx5_ifc_modify_flow_table_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x60];
};

struct mlx5_ifc_query_flow_table_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 vhca_tunnel_id[0x10];
	u8 op_mod[0x10];

	u8 other_vport[0x1];
	u8 reserved_at_41[0xf];
	u8 vport_number[0x10];

	u8 reserved_at_60[0x20];

	u8 table_type[0x8];
	u8 reserved_at_88[0x18];

	u8 reserved_at_a0[0x8];
	u8 table_id[0x18];

	u8 reserved_at_c0[0x140];
};

struct mlx5_ifc_query_flow_table_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x80];

	struct mlx5_ifc_flow_table_context_bits flow_table_context;
};

struct mlx5_ifc_create_flow_group_in_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x10];
	u8 reserved_at_20[0x20];
	u8 other_vport[0x1];
	u8 reserved_at_41[0xf];
	u8 vport_number[0x10];
	u8 reserved_at_60[0x20];
	u8 table_type[0x8];
	u8 reserved_at_88[0x18];
	u8 reserved_at_a0[0x8];
	u8 table_id[0x18];
	u8 reserved_at_c0[0x1f40];
};

struct mlx5_ifc_create_flow_group_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x8];
	u8 group_id[0x18];
	u8 reserved_at_60[0x20];
};

enum {
	MLX5_IFC_MODIFY_FLOW_TABLE_MISS_ACTION = 1 << 0,
	MLX5_IFC_MODIFY_FLOW_TABLE_RTC_ID = 1 << 1,
};

enum {
	MLX5_IFC_MODIFY_FLOW_TABLE_MISS_ACTION_DEFAULT = 0,
	MLX5_IFC_MODIFY_FLOW_TABLE_MISS_ACTION_GOTO_TBL = 1,
};

enum mlx5_ifc_flow_context_action {
	MLX5_IFC_FLOW_CONTEXT_ACTION_DROP = 1 << 1,
	MLX5_IFC_FLOW_CONTEXT_ACTION_FWD_DEST = 1 << 2,
	MLX5_IFC_FLOW_CONTEXT_ACTION_REFORMAT = 1 << 4,
	MLX5_IFC_FLOW_CONTEXT_ACTION_DECRYPT = 1 << 12,
	MLX5_IFC_FLOW_CONTEXT_ACTION_ENCRYPT = 1 << 13,
};

enum mlx5_ifc_packet_reformat_context_reformat_type {
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_L2_TO_L2_TUNNEL = 0x2,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_L2_TO_L3_TUNNEL = 0x4,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_ADD_ESP_TRANSPORT_OVER_IPV4 = 0x5,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_L2_TO_L3_ESP_TUNNEL = 0x6,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_ADD_ESP_TRANSPORT_OVER_UDPV4 = 0x7,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_DEL_ESP_TRANSPORT = 0x8,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_L3_ESP_TUNNEL_TO_L2 = 0x9,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_DEL_ESP_TRANSPORT_OVER_UDP = 0xA,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_ADD_ESP_TRANSPORT_OVER_IPV6 = 0xB,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_ADD_ESP_TRANSPORT_OVER_UDPV6 = 0xC,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_ADD_NISP_TNL = 0xD,
	MLX5_IFC_PACKET_REFORMAT_CONTEXT_REFORMAT_TYPE_REMOVE_NISP_TNL = 0xE,
};

struct mlx5_ifc_set_fte_in_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 other_vport[0x1];
	u8 reserved_at_41[0xf];
	u8 vport_number[0x10];
	u8 reserved_at_60[0x20];
	u8 table_type[0x8];
	u8 reserved_at_88[0x18];
	u8 reserved_at_a0[0x8];
	u8 table_id[0x18];
	u8 ignore_flow_level[0x1];
	u8 reserved_at_c1[0x17];
	u8 modify_enable_mask[0x8];
	u8 reserved_at_e0[0x20];
	u8 flow_index[0x20];
	u8 reserved_at_120[0xe0];
	struct mlx5_ifc_flow_context_bits flow_context;
};

struct mlx5_ifc_set_fte_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_extended_dest_format_bits {
	struct mlx5_ifc_dest_format_bits destination_entry;

	u8 packet_reformat_id[0x20];

	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_packet_reformat_context_in_bits {
	u8 reformat_type[0x8];
	u8 reserved_at_8[0x4];
	u8 reformat_param_0[0x4];
	u8 reserved_at_16[0x6];
	u8 reformat_data_size[0xa];

	u8 reformat_param_1[0x8];
	u8 reserved_at_28[0x8];
	u8 reformat_data[6][0x8];

	u8 more_reformat_data[][0x8];
};

struct mlx5_ifc_alloc_packet_reformat_context_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0xa0];

	u8 packet_reformat_context[];
};

struct mlx5_ifc_alloc_packet_reformat_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 packet_reformat_id[0x20];

	u8 reserved_at_60[0x20];
};

enum {
	MLX5_SQC_STATE_RST  = 0x0,
	MLX5_SQC_STATE_RDY  = 0x1,
	MLX5_SQC_STATE_ERR  = 0x3,
};

enum {
	MLX5_WQ_TYPE_LINKED_LIST                = 0x0,
	MLX5_WQ_TYPE_CYCLIC                     = 0x1,
	MLX5_WQ_TYPE_LINKED_LIST_STRIDING_RQ    = 0x2,
	MLX5_WQ_TYPE_CYCLIC_STRIDING_RQ         = 0x3,
};

struct mlx5_ifc_wq_bits {
	u8 wq_type[0x4];
	u8 wq_signature[0x1];
	u8 end_padding_mode[0x2];
	u8 cd_slave[0x1];
	u8 reserved_at_8[0x18];
	u8 hds_skip_first_sge[0x1];
	u8 log2_hds_buf_size[0x3];
	u8 reserved_at_24[0x7];
	u8 page_offset[0x5];
	u8 lwm[0x10];
	u8 reserved_at_40[0x8];
	u8 pd[0x18];
	u8 reserved_at_60[0x8];
	u8 uar_page[0x18];
	u8 dbr_addr[0x40];
	u8 hw_counter[0x20];
	u8 sw_counter[0x20];
	u8 reserved_at_100[0xc];
	u8 log_wq_stride[0x4];
	u8 reserved_at_110[0x3];
	u8 log_wq_pg_sz[0x5];
	u8 reserved_at_118[0x3];
	u8 log_wq_sz[0x5];
	u8 dbr_umem_valid[0x1];
	u8 wq_umem_valid[0x1];
	u8 reserved_at_122[0x1];
	u8 log_hairpin_num_packets[0x5];
	u8 reserved_at_128[0x3];
	u8 log_hairpin_data_sz[0x5];
	u8 reserved_at_130[0x4];
	u8 single_wqe_log_num_of_strides[0x4];
	u8 two_byte_shift_en[0x1];
	u8 reserved_at_139[0x4];
	u8 single_stride_log_num_of_bytes[0x3];
	u8 dbr_umem_id[0x20];
	u8 wq_umem_id[0x20];
	u8 wq_umem_offset[0x40];
	u8 reserved_at_1c0[0x440];
};

struct mlx5_ifc_sqc_bits {
	u8 rlky[0x1];
	u8 cd_master[0x1];
	u8 fre[0x1];
	u8 flush_in_error_en[0x1];
	u8 allow_multi_pkt_send_wqe[0x1];
	u8 min_wqe_inline_mode[0x3];
	u8 state[0x4];
	u8 reg_umr[0x1];
	u8 allow_swp[0x1];
	u8 hairpin[0x1];
	u8 non_wire[0x1];
	u8 static_sq_wq[0x1];
	u8 reserved_at_11[0x4];
	u8 hairpin_wq_buffer_type[0x3];
	u8 reserved_at_18[0x2];
	u8 ts_format[0x02];
	u8 reserved_at_1c[0x4];
	u8 reserved_at_20[0x8];
	u8 user_index[0x18];
	u8 reserved_at_40[0x8];
	u8 cqn[0x18];
	u8 reserved_at_60[0x8];
	u8 hairpin_peer_rq[0x18];
	u8 reserved_at_80[0x10];
	u8 hairpin_peer_vhca[0x10];
	u8 reserved_at_a0[0x50];
	u8 packet_pacing_rate_limit_index[0x10];
	u8 tis_lst_sz[0x10];
	u8 reserved_at_110[0x10];
	u8 reserved_at_120[0x40];
	u8 reserved_at_160[0x8];
	u8 tis_num_0[0x18];
	struct mlx5_ifc_wq_bits wq;
};

struct mlx5_ifc_modify_sq_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_modify_sq_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 sq_state[0x4];
	u8 reserved_at_44[0x4];
	u8 sqn[0x18];
	u8 reserved_at_60[0x20];
	u8 modify_bitmask[0x40];
	u8 reserved_at_c0[0x40];
	struct mlx5_ifc_sqc_bits ctx;
};

struct mlx5_ifc_create_sq_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x8];
	u8 sqn[0x18];
	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_create_sq_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 reserved_at_40[0xc0];
	struct mlx5_ifc_sqc_bits ctx;
};

enum mlx5_ifc_cross_vhca_object_to_object_supported_types {
	MLX5_CROSS_VHCA_OBJ_TO_OBJ_TYPE_STC_TO_TIR = 1 << 10,
	MLX5_CROSS_VHCA_OBJ_TO_OBJ_TYPE_STC_TO_FT = 1 << 11,
	MLX5_CROSS_VHCA_OBJ_TO_OBJ_TYPE_FT_TO_FT = 1 << 12,
	MLX5_CROSS_VHCA_OBJ_TO_OBJ_TYPE_FT_TO_RTC = 1 << 13,
};

enum mlx5_ifc_cross_vhca_allowed_objects_types {
	MLX5_CROSS_VHCA_ALLOWED_OBJS_TIR = 1 << 0x8,
	MLX5_CROSS_VHCA_ALLOWED_OBJS_FT = 1 << 0x9,
	MLX5_CROSS_VHCA_ALLOWED_OBJS_RTC = 1 << 0xa,
};

enum {
	MLX5_HCA_CAP_OPMOD_GET_MAX   = 0,
	MLX5_HCA_CAP_OPMOD_GET_CUR   = 1,
};

enum {
	MLX5_GET_HCA_CAP_OP_MOD_GENERAL_DEVICE = 0x0 << 1,
	MLX5_GET_HCA_CAP_OP_MOD_ROCE = 0x4 << 1,
	MLX5_GET_HCA_CAP_OP_MOD_NIC_FLOW_TABLE = 0x7 << 1,
	MLX5_GET_HCA_CAP_OP_MOD_ESW_FLOW_TABLE = 0x8 << 1,
	MLX5_SET_HCA_CAP_OP_MOD_ESW = 0x9 << 1,
	MLX5_GET_HCA_CAP_OP_MOD_WQE_BASED_FLOW_TABLE = 0x1b << 1,
	MLX5_GET_HCA_CAP_OP_MOD_PARSE_GRAPH_NODE_CAP = 0x1C << 1,
	MLX5_GET_HCA_CAP_OP_MOD_GENERAL_DEVICE_2 = 0x20 << 1,
};

/* The supported timestamp formats reported in HCA attributes. */
enum {
	MLX5_HCA_CAP_TIMESTAMP_FORMAT_FR = 0x0,
	MLX5_HCA_CAP_TIMESTAMP_FORMAT_RT = 0x1,
	MLX5_HCA_CAP_TIMESTAMP_FORMAT_FR_RT = 0x2,
};

/* The timestamp format attributes to configure queues (RQ/SQ/QP). */
enum {
	MLX5_QPC_TIMESTAMP_FORMAT_FREE_RUNNING = 0x0,
	MLX5_QPC_TIMESTAMP_FORMAT_DEFAULT = 0x1,
	MLX5_QPC_TIMESTAMP_FORMAT_REAL_TIME = 0x2,
};

struct mlx5_ifc_cmd_hca_cap_bits {
	u8 access_other_hca_roce[0x1];
	u8 alloc_flow_counter_pd[0x1];
	u8 flow_counter_access_aso[0x1];
	u8 query_match_sample_info[0x1];
	u8 reserved_at_4[0x4];
	u8 flow_counter_access_aso_opc_mod[0x8];
	u8 reserved_at_10[0xf];
	u8 vhca_resource_manager[0x1];
	u8 hca_cap_2[0x1];
	u8 reserved_at_21[0xf];
	u8 vhca_id[0x10];
	u8 reserved_at_40[0x20];
	u8 reserved_at_60[0x3];
	u8 log_regexp_scatter_gather_size[0x5];
	u8 reserved_at_68[0x3];
	u8 log_dma_mmo_size[0x5];
	u8 reserved_at_70[0x3];
	u8 log_compress_mmo_size[0x5];
	u8 decompress_lz4_data_only_v2[0x1];
	u8 decompress_lz4_no_checksum_v2[0x1];
	u8 decompress_lz4_checksum_v2[0x1];
	u8 log_decompress_mmo_size[0x5];
	u8 log_max_srq_sz[0x8];
	u8 log_max_qp_sz[0x8];
	u8 reserved_at_90[0x9];
	u8 wqe_index_ignore_cap[0x1];
	u8 dynamic_qp_allocation[0x1];
	u8 log_max_qp[0x5];
	u8 reserved_at_a0[0x4];
	u8 regexp_num_of_engines[0x4];
	u8 reserved_at_a8[0x1];
	u8 reg_c_preserve[0x1];
	u8 reserved_at_aa[0x1];
	u8 log_max_srq[0x5];
	u8 psp_old[0x1];
	u8 reserved_at_b1[0xa];
	u8 scatter_fcs_w_decap_disable[0x1];
	u8 reserved_at_bc[0x4];
	u8 reserved_at_c0[0x8];
	u8 log_max_cq_sz[0x8];
	u8 reserved_at_d0[0x2];
	u8 access_register_user[0x1];
	u8 reserved_at_d3[0x8];
	u8 log_max_cq[0x5];
	u8 log_max_eq_sz[0x8];
	u8 relaxed_ordering_write[0x1];
	u8 relaxed_ordering_read[0x1];
	u8 log_max_mkey[0x6];
	u8 reserved_at_f0[0x8];
	u8 dump_fill_mkey[0x1];
	u8 reserved_at_f9[0x3];
	u8 log_max_eq[0x4];
	u8 max_indirection[0x8];
	u8 fixed_buffer_size[0x1];
	u8 log_max_mrw_sz[0x7];
	u8 force_teardown[0x1];
	u8 reserved_at_111[0x1];
	u8 log_max_bsf_list_size[0x6];
	u8 umr_extended_translation_offset[0x1];
	u8 null_mkey[0x1];
	u8 log_max_klm_list_size[0x6];
	u8 non_wire_sq[0x1];
	u8 reserved_at_121[0x9];
	u8 log_max_ra_req_dc[0x6];
	u8 reserved_at_130[0x3];
	u8 log_max_static_sq_wq[0x5];
	u8 reserved_at_138[0x2];
	u8 log_max_ra_res_dc[0x6];
	u8 reserved_at_140[0xa];
	u8 log_max_ra_req_qp[0x6];
	u8 rtr2rts_qp_counters_set_id[0x1];
	u8 rts2rts_udp_sport[0x1];
	u8 rts2rts_lag_tx_port_affinity[0x1];
	u8 dma_mmo_sq[0x1];
	u8 compress_min_block_size[0x4];
	u8 compress_mmo_sq[0x1];
	u8 decompress_mmo_sq[0x1];
	u8 log_max_ra_res_qp[0x6];
	u8 end_pad[0x1];
	u8 cc_query_allowed[0x1];
	u8 cc_modify_allowed[0x1];
	u8 start_pad[0x1];
	u8 cache_line_128byte[0x1];
	u8 reserved_at_165[0xa];
	u8 qcam_reg[0x1];
	u8 gid_table_size[0x10];
	u8 out_of_seq_cnt[0x1];
	u8 vport_counters[0x1];
	u8 retransmission_q_counters[0x1];
	u8 debug[0x1];
	u8 modify_rq_counter_set_id[0x1];
	u8 rq_delay_drop[0x1];
	u8 max_qp_cnt[0xa];
	u8 pkey_table_size[0x10];
	u8 vport_group_manager[0x1];
	u8 vhca_group_manager[0x1];
	u8 ib_virt[0x1];
	u8 eth_virt[0x1];
	u8 vnic_env_queue_counters[0x1];
	u8 ets[0x1];
	u8 nic_flow_table[0x1];
	u8 eswitch_manager[0x1];
	u8 device_memory[0x1];
	u8 mcam_reg[0x1];
	u8 pcam_reg[0x1];
	u8 local_ca_ack_delay[0x5];
	u8 port_module_event[0x1];
	u8 enhanced_error_q_counters[0x1];
	u8 ports_check[0x1];
	u8 reserved_at_1b3[0x1];
	u8 disable_link_up[0x1];
	u8 beacon_led[0x1];
	u8 port_type[0x2];
	u8 num_ports[0x8];
	u8 reserved_at_1c0[0x1];
	u8 pps[0x1];
	u8 pps_modify[0x1];
	u8 log_max_msg[0x5];
	u8 reserved_at_1c8[0x4];
	u8 max_tc[0x4];
	u8 temp_warn_event[0x1];
	u8 dcbx[0x1];
	u8 general_notification_event[0x1];
	u8 reserved_at_1d3[0x2];
	u8 fpga[0x1];
	u8 rol_s[0x1];
	u8 rol_g[0x1];
	u8 reserved_at_1d8[0x1];
	u8 wol_s[0x1];
	u8 wol_g[0x1];
	u8 wol_a[0x1];
	u8 wol_b[0x1];
	u8 wol_m[0x1];
	u8 wol_u[0x1];
	u8 wol_p[0x1];
	u8 stat_rate_support[0x10];
	u8 reserved_at_1ef[0xb];
	u8 wqe_based_flow_table_update_cap[0x1];
	u8 cqe_version[0x4];
	u8 compact_address_vector[0x1];
	u8 striding_rq[0x1];
	u8 reserved_at_202[0x1];
	u8 ipoib_enhanced_offloads[0x1];
	u8 ipoib_basic_offloads[0x1];
	u8 reserved_at_205[0x1];
	u8 repeated_block_disabled[0x1];
	u8 umr_modify_entity_size_disabled[0x1];
	u8 umr_modify_atomic_disabled[0x1];
	u8 umr_indirect_mkey_disabled[0x1];
	u8 umr_fence[0x2];
	u8 reserved_at_20c[0x3];
	u8 drain_sigerr[0x1];
	u8 cmdif_checksum[0x2];
	u8 sigerr_cqe[0x1];
	u8 reserved_at_213[0x1];
	u8 wq_signature[0x1];
	u8 sctr_data_cqe[0x1];
	u8 reserved_at_216[0x1];
	u8 sho[0x1];
	u8 tph[0x1];
	u8 rf[0x1];
	u8 dct[0x1];
	u8 qos[0x1];
	u8 eth_net_offloads[0x1];
	u8 roce[0x1];
	u8 atomic[0x1];
	u8 reserved_at_21f[0x1];
	u8 cq_oi[0x1];
	u8 cq_resize[0x1];
	u8 cq_moderation[0x1];
	u8 reserved_at_223[0x3];
	u8 cq_eq_remap[0x1];
	u8 pg[0x1];
	u8 block_lb_mc[0x1];
	u8 reserved_at_229[0x1];
	u8 scqe_break_moderation[0x1];
	u8 cq_period_start_from_cqe[0x1];
	u8 cd[0x1];
	u8 reserved_at_22d[0x1];
	u8 apm[0x1];
	u8 vector_calc[0x1];
	u8 umr_ptr_rlky[0x1];
	u8 imaicl[0x1];
	u8 reserved_at_232[0x4];
	u8 qkv[0x1];
	u8 pkv[0x1];
	u8 set_deth_sqpn[0x1];
	u8 reserved_at_239[0x3];
	u8 xrc[0x1];
	u8 ud[0x1];
	u8 uc[0x1];
	u8 rc[0x1];
	u8 uar_4k[0x1];
	u8 reserved_at_241[0x8];
	u8 regexp_params[0x1];
	u8 uar_sz[0x6];
	u8 port_selection_cap[0x1];
	u8 reserved_at_251[0x7];
	u8 log_pg_sz[0x8];
	u8 bf[0x1];
	u8 driver_version[0x1];
	u8 pad_tx_eth_packet[0x1];
	u8 reserved_at_263[0x8];
	u8 log_bf_reg_size[0x5];
	u8 reserved_at_270[0xb];
	u8 lag_master[0x1];
	u8 num_lag_ports[0x4];
	u8 reserved_at_280[0x10];
	u8 max_wqe_sz_sq[0x10];
	u8 reserved_at_2a0[0xa];
	u8 psp[0x1];
	u8 reserved_at_2ab[0x1];
	u8 regexp_mmo_sq[0x1];
	u8 regexp_version[0x3];
	u8 max_wqe_sz_rq[0x10];
	u8 max_flow_counter_31_16[0x10];
	u8 max_wqe_sz_sq_dc[0x10];
	u8 reserved_at_2e0[0x7];
	u8 max_qp_mcg[0x19];
	u8 reserved_at_300[0x10];
	u8 flow_counter_bulk_alloc[0x08];
	u8 log_max_mcg[0x8];
	u8 reserved_at_320[0x3];
	u8 log_max_transport_domain[0x5];
	u8 reserved_at_328[0x3];
	u8 log_max_pd[0x5];
	u8 reserved_at_330[0xb];
	u8 log_max_xrcd[0x5];
	u8 nic_receive_steering_discard[0x1];
	u8 receive_discard_vport_down[0x1];
	u8 transmit_discard_vport_down[0x1];
	u8 reserved_at_343[0x5];
	u8 log_max_flow_counter_bulk[0x8];
	u8 max_flow_counter_15_0[0x10];
	u8 modify_tis[0x1];
	u8 flow_counters_dump[0x1];
	u8 reserved_at_360[0x1];
	u8 log_max_rq[0x5];
	u8 reserved_at_368[0x3];
	u8 log_max_sq[0x5];
	u8 reserved_at_370[0x3];
	u8 log_max_tir[0x5];
	u8 reserved_at_378[0x3];
	u8 log_max_tis[0x5];
	u8 basic_cyclic_rcv_wqe[0x1];
	u8 reserved_at_381[0x1];
	u8 mem_rq_rmp[0x1];
	u8 log_max_rmp[0x5];
	u8 reserved_at_388[0x3];
	u8 log_max_rqt[0x5];
	u8 reserved_at_390[0x3];
	u8 log_max_rqt_size[0x5];
	u8 reserved_at_398[0x3];
	u8 log_max_tis_per_sq[0x5];
	u8 ext_stride_num_range[0x1];
	u8 reserved_at_3a1[0x2];
	u8 log_max_stride_sz_rq[0x5];
	u8 reserved_at_3a8[0x3];
	u8 log_min_stride_sz_rq[0x5];
	u8 reserved_at_3b0[0x3];
	u8 log_max_stride_sz_sq[0x5];
	u8 reserved_at_3b8[0x3];
	u8 log_min_stride_sz_sq[0x5];
	u8 hairpin[0x1];
	u8 reserved_at_3c1[0x2];
	u8 log_max_hairpin_queues[0x5];
	u8 reserved_at_3c8[0x3];
	u8 log_max_hairpin_wq_data_sz[0x5];
	u8 reserved_at_3d0[0x3];
	u8 log_max_hairpin_num_packets[0x5];
	u8 reserved_at_3d8[0x3];
	u8 log_max_wq_sz[0x5];
	u8 nic_vport_change_event[0x1];
	u8 disable_local_lb_uc[0x1];
	u8 disable_local_lb_mc[0x1];
	u8 log_min_hairpin_wq_data_sz[0x5];
	u8 reserved_at_3e8[0x3];
	u8 log_max_vlan_list[0x5];
	u8 reserved_at_3f0[0x1];
	u8 aes_xts_single_block_le_tweak[1];
	u8 aes_xts_multi_block_be_tweak[1];
	u8 log_max_current_mc_list[0x5];
	u8 reserved_at_3f8[0x3];
	u8 log_max_current_uc_list[0x5];
	u8 general_obj_types[0x40];
	u8 sq_ts_format[0x2];
	u8 rq_ts_format[0x2];
	u8 steering_format_version[0x4];
	u8 reserved_at_448[0x18];
	u8 reserved_at_460[0x8];
	u8 aes_xts[0x1];
	u8 crypto[0x1];
	u8 ipsec_offload[0x1];
	u8 reserved_at_46b[0x5];
	u8 max_num_eqs[0x10];
	u8 reserved_at_480[0x3];
	u8 log_max_l2_table[0x5];
	u8 reserved_at_488[0x8];
	u8 log_uar_page_sz[0x10];
	u8 reserved_at_4a0[0x20];
	u8 device_frequency_mhz[0x20];
	u8 device_frequency_khz[0x20];
	u8 reserved_at_500[0x20];
	u8 num_of_uars_per_page[0x20];
	u8 flex_parser_protocols[0x20];
	u8 max_geneve_tlv_options[0x8];
	u8 geneve_tlv_sample[0x1];
	u8 geneve_tlv_option_offset[0x1];
	u8 reserved_at_56a[0x1];
	u8 max_geneve_tlv_option_data_len[0x5];
	u8 flex_parser_header_modify[0x1];
	u8 reserved_at_571[0x2];
	u8 log_max_guaranteed_connections[0x5];
	u8 driver_version_before_init_hca[0x1];
	u8 adv_virtualization[0x1];
	u8 reserved_at_57a[0x1];
	u8 log_max_dct_connections[0x5];
	u8 log_max_atomic_size_qp[0x8];
	u8 reserved_at_587[0x3];
	u8 log_max_dci_stream_channels[0x5];
	u8 reserved_at_58f[0x3];
	u8 log_max_dci_errored_streams[0x5];
	u8 log_max_atomic_dize_dc[0x8];
	u8 max_multi_user_ggroup_size[0x10];
	u8 enhanced_cqe_compression[0x1];
	u8 reserved_at_5b0[0x1];
	u8 crossing_vhca_mkey[0x1];
	u8 log_max_dek[0x5];
	u8 reserved_at_5b7[0x1];
	u8 mini_cqe_resp_l3_l4_tag[0x1];
	u8 mini_cqe_resp_flow_tag[0x1];
	u8 reserved_at_5ba[0x1];
	u8 mini_cqe_resp_stride_index[0x1];
	u8 cqe_128_always[0x1];
	u8 cqe_compression_128[0x1];
	u8 cqe_compression[0x1];
	u8 cqe_compression_timeout[0x10];
	u8 cqe_compression_max_num[0x10];
	u8 reserved_at_5e0[0x8];
	u8 flex_parser_id_gtpu_dw_0[0x4];
	u8 reserved_at_5ec[0x4];
	u8 tag_matching[0x1];
	u8 rndv_offload_rc[0x1];
	u8 rndv_offload_dc[0x1];
	u8 log_tag_matching_list_sz[0x5];
	u8 reserved_at_5f8[0x3];
	u8 log_max_xrq[0x5];
	u8 affiliate_nic_vport_criteria[0x8];
	u8 native_port_num[0x8];
	u8 num_vhca_ports[0x8];
	u8 flex_parser_id_gtpu_teid[0x4];
	u8 reserved_at_61c[0x2];
	u8 sw_owner_id[0x1];
	u8 reserved_at_61f[0x6C];
	u8 wait_on_data[0x1];
	u8 wait_on_time[0x1];
	u8 reserved_at_68d[0x37];
	u8 flex_parser_id_geneve_opt_0[0x4];
	u8 flex_parser_id_icmp_dw1[0x4];
	u8 flex_parser_id_icmp_dw0[0x4];
	u8 flex_parser_id_icmpv6_dw1[0x4];
	u8 flex_parser_id_icmpv6_dw0[0x4];
	u8 flex_parser_id_outer_first_mpls_over_gre[0x4];
	u8 flex_parser_id_outer_first_mpls_over_udp_label[0x4];
	u8 reserved_at_6e0[0x20];
	u8 flex_parser_id_gtpu_dw_2[0x4];
	u8 flex_parser_id_gtpu_first_ext_dw_0[0x4];
	u8 reserved_at_708[0x39];
	u8 qp_mmo_type[0x1];
	u8 reserved_at_742[0x6];
	u8 dma_mmo_qp[0x1];
	u8 regexp_mmo_qp[0x1];
	u8 compress_mmo_qp[0x1];
	u8 decompress_deflate_v1[0x1];
	u8 reserved_at_74c[0x4];
	u8 decompress_deflate_v2[0x1];
	u8 reserved_at_751[0xf];
	u8 reserved_at_760[0x3];
	u8 log_max_num_header_modify_argument[0x5];
	u8 log_header_modify_argument_granularity_offset[0x4];
	u8 log_header_modify_argument_granularity[0x4];
	u8 reserved_at_770[0x3];
	u8 log_header_modify_argument_max_alloc[0x5];
	u8 reserved_at_778[0x8];
	u8 reserved_at_780[0x40];
	u8 match_definer_format_supported[0x40];
};

enum {
	MLX5_GENERATE_WQE_TYPE_FLOW_UPDATE = 1 << 1,
};

struct mlx5_ifc_cmd_hca_cap_2_bits {
	u8 reserved_at_0[0x80]; /* End of DW4. */
	u8 reserved_at_80[0x3];
	u8 max_num_prog_sample_field[0x5];
	u8 reserved_at_88[0x3];
	u8 log_max_num_reserved_qpn[0x5];
	u8 reserved_at_90[0x3];
	u8 log_reserved_qpn_granularity[0x5];
	u8 reserved_at_98[0x3];
	u8 log_reserved_qpn_max_alloc[0x5]; /* End of DW5. */
	u8 max_reformat_insert_size[0x8];
	u8 max_reformat_insert_offset[0x8];
	u8 max_reformat_remove_size[0x8];
	u8 max_reformat_remove_offset[0x8]; /* End of DW6. */
	u8 reserved_at_c0[0x3];
	u8 log_min_stride_wqe_sz[0x5];
	u8 reserved_at_c8[0x3];
	u8 log_conn_track_granularity[0x5];
	u8 reserved_at_d0[0x3];
	u8 log_conn_track_max_alloc[0x5];
	u8 reserved_at_d8[0x3];
	u8 log_max_conn_track_offload[0x5]; /* End of DW7. */
	u8 cross_vhca_object_to_object_supported[0x20];
	u8 allowed_object_for_other_vhca_access_high[0x20];
	u8 allowed_object_for_other_vhca_access[0x20];
	u8 reserved_at_140[0x20];
	u8 reserved_at_160[0x3];
	u8 hairpin_sq_wqe_bb_size[0x5];
	u8 hairpin_sq_wq_in_host_mem[0x1];
	u8 hairpin_data_buffer_locked[0x1];
	u8 reserved_at_16a[0x16];
	u8 reserved_at_180[0x20];
	u8 reserved_at_1a0[0xa];
	u8 format_select_dw_8_6_ext[0x1];
	u8 reserved_at_1ac[0x15];
	u8 general_obj_types_127_64[0x40];
	u8 reserved_at_200[0x53];
	u8 flow_counter_bulk_log_max_alloc[0x5];
	u8 reserved_at_258[0x3];
	u8 flow_counter_bulk_log_granularity[0x5];
	u8 reserved_at_260[0x20];
	u8 format_select_dw_gtpu_dw_0[0x8];
	u8 format_select_dw_gtpu_dw_1[0x8];
	u8 format_select_dw_gtpu_dw_2[0x8];
	u8 format_select_dw_gtpu_first_ext_dw_0[0x8];
	u8 generate_wqe_type[0x20];
	u8 reserved_at_2c0[0x150];
	u8 return_reg_id[0x10];
	u8 reserved_at_420[0x18];
	u8 encap_entropy_hash_type[0x4];
	u8 flow_table_hash_type[0x4];
	u8 reserved_at_440[0x3c0];
};

struct mlx5_ifc_ft_prop_layout_bits {
	u8 ft_support[0x1];
	u8 flow_tag[0x1];
	u8 flow_counter[0x1];
	u8 flow_modify_en[0x1];
	u8 modify_root[0x1];
	u8 identified_miss_table[0x1];
	u8 flow_table_modify[0x1];
	u8 reformat[0x1];
	u8 decap[0x1];
	u8 reset_root_to_default[0x1];
	u8 pop_vlan[0x1];
	u8 push_vlan[0x1];
	u8 fpga_vendor_acceleration[0x1];
	u8 pop_vlan_2[0x1];
	u8 push_vlan_2[0x1];
	u8 reformat_and_vlan_action[0x1];
	u8 modify_and_vlan_action[0x1];
	u8 sw_owner[0x1];
	u8 reformat_l3_tunnel_to_l2[0x1];
	u8 reformat_l2_to_l3_tunnel[0x1];
	u8 reformat_and_modify_action[0x1];
	u8 reserved_at_15[0x9];
	u8 sw_owner_v2[0x1];
	u8 reserved_at_1f[0x1];
	u8 reserved_at_20[0x2];
	u8 log_max_ft_size[0x6];
	u8 log_max_modify_header_context[0x8];
	u8 max_modify_header_actions[0x8];
	u8 max_ft_level[0x8];
	u8 reformat_add_esp_transport[0x1];
	u8 reformat_l2_to_l3_esp_tunnel[0x1];
	u8 reformat_add_esp_transport_over_udp[0x1];
	u8 reformat_del_esp_transport[0x1];
	u8 reformat_l3_esp_tunnel_to_l2[0x1];
	u8 reformat_del_esp_transport_over_udp[0x1];
	u8 reserved_at_46[0x2];
	u8 log_max_ft_sampler_num[8];
	u8 metadata_reg_b_width[0x8];
	u8 metadata_reg_a_width[0x8];
	u8 reserved_at_60[0xa];
	u8 reparse[0x1];
	u8 reserved_at_6b[0x1];
	u8 cross_vhca_object[0x1];
	u8 reformat_l2_to_l3_audp_tunnel[0x1];
	u8 reformat_l3_audp_tunnel_to_l2[0x1];
	u8 ignore_flow_level_rtc_valid[0x1];
	u8 reserved_at_70[0x8];
	u8 log_max_ft_num[0x8];
	u8 reserved_at_80[0x10];
	u8 log_max_flow_counter[0x8];
	u8 log_max_destination[0x8];
	u8 reserved_at_a0[0x18];
	u8 log_max_flow[0x8];
	u8 reserved_at_c0[0x140];
};

struct mlx5_ifc_ft_fields_support_bits {
	/* set_action_field_support */
	u8 outer_dmac[0x1];
	u8 outer_smac[0x1];
	u8 outer_ether_type[0x1];
	u8 reserved_at_3[0x1];
	u8 outer_first_prio[0x1];
	u8 outer_first_cfi[0x1];
	u8 outer_first_vid[0x1];
	u8 reserved_at_7[0x1];
	u8 outer_second_prio[0x1];
	u8 outer_second_cfi[0x1];
	u8 outer_second_vid[0x1];
	u8 reserved_at_b[0x1];
	u8 outer_sip[0x1];
	u8 outer_dip[0x1];
	u8 outer_frag[0x1];
	u8 outer_ip_protocol[0x1];
	u8 outer_ip_ecn[0x1];
	u8 outer_ip_dscp[0x1];
	u8 outer_udp_sport[0x1];
	u8 outer_udp_dport[0x1];
	u8 outer_tcp_sport[0x1];
	u8 outer_tcp_dport[0x1];
	u8 outer_tcp_flags[0x1];
	u8 outer_gre_protocol[0x1];
	u8 outer_gre_key[0x1];
	u8 outer_vxlan_vni[0x1];
	u8 reserved_at_1a[0x5];
	u8 source_eswitch_port[0x1]; /* end of DW0 */
	u8 inner_dmac[0x1];
	u8 inner_smac[0x1];
	u8 inner_ether_type[0x1];
	u8 reserved_at_23[0x1];
	u8 inner_first_prio[0x1];
	u8 inner_first_cfi[0x1];
	u8 inner_first_vid[0x1];
	u8 reserved_at_27[0x1];
	u8 inner_second_prio[0x1];
	u8 inner_second_cfi[0x1];
	u8 inner_second_vid[0x1];
	u8 reserved_at_2b[0x1];
	u8 inner_sip[0x1];
	u8 inner_dip[0x1];
	u8 inner_frag[0x1];
	u8 inner_ip_protocol[0x1];
	u8 inner_ip_ecn[0x1];
	u8 inner_ip_dscp[0x1];
	u8 inner_udp_sport[0x1];
	u8 inner_udp_dport[0x1];
	u8 inner_tcp_sport[0x1];
	u8 inner_tcp_dport[0x1];
	u8 inner_tcp_flags[0x1];
	u8 reserved_at_37[0x9]; /* end of DW1 */
	u8 reserved_at_40[0x20]; /* end of DW2 */
	u8 reserved_at_60[0x18];
	union {
		struct {
			u8 metadata_reg_c_7[0x1];
			u8 metadata_reg_c_6[0x1];
			u8 metadata_reg_c_5[0x1];
			u8 metadata_reg_c_4[0x1];
			u8 metadata_reg_c_3[0x1];
			u8 metadata_reg_c_2[0x1];
			u8 metadata_reg_c_1[0x1];
			u8 metadata_reg_c_0[0x1];
		};
		u8 metadata_reg_c_x[0x8];
	}; /* end of DW3 */
	/* set_action_field_support_2 */
	u8 reserved_at_80[0x37];
	u8 outer_ipv6_traffic_class[0x1];
	u8 reserved_at_B8[0x48];
	/* add_action_field_support */
	u8 reserved_at_100[0x80];
	/* add_action_field_support_2 */
	u8 reserved_at_180[0x80];
	/* copy_action_field_support */
	u8 reserved_at_200[0x80];
	/* copy_action_field_support_2 */
	u8 reserved_at_280[0x80];
	u8 reserved_at_300[0x100];
};

struct mlx5_ifc_ft_fields_support_2_bits {
	u8 reserved_at_0[0x8];
	u8 inner_mpls[0x1];
	u8 outer_mpls[0x1];
	u8 lag_rx_port_affinity[0x1];
	u8 reserved_at_c[0x2];
	u8 hash_result[0x1];
	u8 reserved_at_e[0x1];
	u8 tunnel_header_2_3[0x1];
	u8 tunnel_header_0_1[0x1];
	u8 macsec_syndrome[0x1];
	u8 macsec_tag[0x1];
	u8 outer_lrh_sl[0x1];
	u8 inner_ipv4_ihl[0x1];
	u8 outer_ipv4_ihl[0x1];
	u8 psp_syndrome[0x1];
	u8 inner_l3_ok[0x1];
	u8 inner_l4_ok[0x1];
	u8 outer_l3_ok[0x1];
	u8 outer_l4_ok[0x1];
	u8 psp_header[0x1];
	u8 inner_ipv4_checksum_ok[0x1];
	u8 inner_l4_checksum_ok[0x1];
	u8 outer_ipv4_checksum_ok[0x1];
	u8 outer_l4_checksum_ok[0x1]; /* end of DW0 */
	u8 reserved_at_20[0x17];
	u8 outer_ipv6_traffic_class[0x1];
	union {
		struct {
			u8 metadata_reg_c_15[0x1];
			u8 metadata_reg_c_14[0x1];
			u8 metadata_reg_c_13[0x1];
			u8 metadata_reg_c_12[0x1];
			u8 metadata_reg_c_11[0x1];
			u8 metadata_reg_c_10[0x1];
			u8 metadata_reg_c_9[0x1];
			u8 metadata_reg_c_8[0x1];
		};
		u8 metadata_reg_c_8_15[0x8];
	}; /* end of DW1 */
	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_flow_table_nic_cap_bits {
	u8 reserved_at_0[0x200];
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_receive;
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_receive_rdma;
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_receive_sniffer;
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_transmit;
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_transmit_rdma;
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_transmit_sniffer;
	u8 reserved_at_e00[0x200];
	struct mlx5_ifc_ft_fields_support_bits ft_header_modify_nic_receive;
	struct mlx5_ifc_ft_fields_support_2_bits ft_field_support_2_nic_receive;
	u8 reserved_at_1480[0x280];
	struct mlx5_ifc_ft_fields_support_2_bits ft_field_support_2_nic_transmit;
	u8 reserved_at_1780[0x480];
	struct mlx5_ifc_ft_fields_support_bits ft_header_modify_nic_transmit;
	u8 reserved_at_2000[0x6000];
};

struct mlx5_ifc_flow_table_esw_cap_bits {
	u8 reserved_at_0[0xd];
	u8 fdb_uplink_hairpin[0x1];
	u8 reserved_at_e[0x1f2];
	struct mlx5_ifc_ft_prop_layout_bits ft_properties_nic_esw_fdb;
	u8 reserved_at_400[0x400];
	struct mlx5_ifc_ft_fields_support_bits ft_header_modify_esw_fdb;
	u8 reserved_at_C00[0x800];
	struct mlx5_ifc_ft_fields_support_2_bits ft_field_support_2_esw_fdb;
	u8 reserved_at_1480[0x6b80];
};

struct mlx5_ifc_esw_cap_bits {
	u8 reserved_at_0[0x1d];
	u8 merged_eswitch[0x1];
	u8 reserved_at_1e[0x2];
	u8 reserved_at_20[0x40];
	u8 esw_manager_vport_number_valid[0x1];
	u8 reserved_at_61[0xf];
	u8 esw_manager_vport_number[0x10];
	u8 reserved_at_80[0x780];
};

struct mlx5_ifc_roce_caps_bits {
	u8 reserved_0[0x1e];
	u8 qp_ts_format[0x2];
	u8 reserved_at_20[0xa0];
	u8 r_roce_max_src_udp_port[0x10];
	u8 r_roce_min_src_udp_port[0x10];
	u8 reserved_at_e0[0x720];
};

enum mlx5_ifc_stc_action_type_cap {
	MLX5_IFC_STC_ACTION_TYPE_CAP_JUMP_FT_FDB_RX = 71,
};

struct mlx5_ifc_wqe_based_flow_table_cap_bits {
	u8 reserved_at_0[0x3];
	u8 log_max_num_ste[0x5];
	u8 reserved_at_8[0x3];
	u8 log_max_num_stc[0x5];
	u8 reserved_at_10[0x3];
	u8 log_max_num_rtc[0x5];
	u8 reserved_at_18[0x3];
	u8 log_max_num_header_modify_pattern[0x5];
	u8 rtc_hash_split_table[0x1];
	u8 rtc_linear_lookup_table[0x1];
	u8 reserved_at_22[0x1];
	u8 stc_alloc_log_granularity[0x5];
	u8 reserved_at_28[0x3];
	u8 stc_alloc_log_max[0x5];
	u8 reserved_at_30[0x3];
	u8 ste_alloc_log_granularity[0x5];
	u8 reserved_at_38[0x3];
	u8 ste_alloc_log_max[0x5];
	u8 reserved_at_40[0xb];
	u8 rtc_reparse_mode[0x5];
	u8 reserved_at_50[0x3];
	u8 rtc_index_mode[0x5];
	u8 reserved_at_58[0x3];
	u8 rtc_log_depth_max[0x5];
	u8 reserved_at_60[0x8];
	u8 max_header_modify_pattern_length[0x8];
	u8 ste_format[0x10];
	u8 stc_action_type_63_0[0x40];
	u8 stc_action_type_127_64[0x40];
	u8 header_insert_type[0x10];
	u8 header_remove_type[0x10];
	u8 trivial_match_definer[0x20];
	u8 reserved_at_140[0x1b];
	u8 rtc_max_num_hash_definer_gen_wqe[0x5];
	u8 reserved_at_160[0x14];
	u8 stc_max_trailer[0x4];
	u8 access_index_mode[0x8];
	u8 reserved_at_180[0x10];
	u8 ste_format_gen_wqe[0x10];
	u8 linear_match_definer_reg_c3[0x20];
	u8 fdb_jump_to_tir_stc[0x1];
	u8 reserved_at_1c1[0x1];
	u8 fdb_unified_en[0x1];
	u8 stc_insert_action_push_esp[0x1];
	u8 reserved_at_1c4[0x1c];
};

struct mlx5_ifc_parse_graph_node_cap_bits {
	u8 node_in[0x20];
	u8 node_out[0x20];
	u8 header_length_mode[0x10];
	u8 sample_offset_mode[0x10];
	u8 max_num_arc_in[0x08];
	u8 max_num_arc_out[0x08];
	u8 max_num_sample[0x08];
	u8 reserved_at_78[0x02];
	u8 parse_graph_override_native_arc[0x1];
	u8 parse_graph_anchor[0x1];
	u8 parse_graph_tlv_option_enable[0x01];
	u8 sample_tunnel_inner2[0x1];
	u8 zero_size_supported[0x1];
	u8 sample_id_in_out[0x1];
	u8 max_base_header_length[0x10];
	u8 reserved_at_90[0x03];
	u8 parse_graph_max_next_header_field_size[0x5];
	u8 max_sample_base_offset[0x08];
	u8 max_next_header_offset[0x10];
	u8 reserved_at_b0[0x03];
	u8 max_num_sample_group[0x5];
	u8 header_length_mask_width[0x08];
	u8 parse_graph_tlv_option_length_mode[0x10];
	u8 reserved_at_1a[0x10];
	u8 reserved_at_1c[0x10];
	u8 parse_graph_tlv_option_length_field_mask_width[0x8];
	u8 parse_graph_tlv_option_max_type_field_size[0x8];
	u8 max_parse_graph_tlv_option_type_field_offset[0x10];
	u8 max_parse_graph_tlv_option_length_base_value[0x10];
	u8 max_parse_graph_tlv_option_offset[0x10];
	u8 max_parse_graph_tlv_option_length_field_offset[0x10];
};

union mlx5_ifc_hca_cap_union_bits {
	struct mlx5_ifc_cmd_hca_cap_bits cmd_hca_cap;
	struct mlx5_ifc_cmd_hca_cap_2_bits cmd_hca_cap_2;
	struct mlx5_ifc_flow_table_nic_cap_bits flow_table_nic_cap;
	struct mlx5_ifc_flow_table_esw_cap_bits flow_table_esw_cap;
	struct mlx5_ifc_esw_cap_bits esw_cap;
	struct mlx5_ifc_roce_caps_bits roce_caps;
	struct mlx5_ifc_wqe_based_flow_table_cap_bits wqe_based_flow_table_cap;
	struct mlx5_ifc_parse_graph_node_cap_bits parse_graph_node_cap;
	u8 reserved_at_0[0x8000];
};

struct mlx5_ifc_query_hca_cap_in_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_query_hca_cap_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x40];
	union mlx5_ifc_hca_cap_union_bits capability;
};

enum {
	MLX5_MATCH_CRITERIA_ENABLE_OUTER_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_INNER_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC2_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC3_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC4_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC5_BIT,
	MLX5_MATCH_CRITERIA_ENABLE_MISC6_BIT,
};

struct mlx5_ifc_dr_match_spec_bits {
	u8 smac_47_16[0x20];
	u8 smac_15_0[0x10];
	u8 ethertype[0x10];
	u8 dmac_47_16[0x20];
	u8 dmac_15_0[0x10];
	u8 first_prio[0x3];
	u8 first_cfi[0x1];
	u8 first_vid[0xc];
	u8 ip_protocol[0x8];
	union {
		u8 tos[0x8];
		struct {
			u8 ip_dscp[0x6];
			u8 ip_ecn[0x2];
		};
	};
	u8 cvlan_tag[0x1];
	u8 svlan_tag[0x1];
	u8 frag[0x1];
	u8 ip_version[0x4];
	u8 tcp_ns[0x1];
	u8 tcp_flags[0x8];
	u8 tcp_sport[0x10];
	u8 tcp_dport[0x10];
	u8 l4_type[0x2];
	u8 reserved_at_c2[0xe];
	u8 ipv4_ihl[0x4];
	u8 l3_ok[0x1];
	u8 l4_ok[0x1];
	u8 ipv4_checksum_ok[0x1];
	u8 l4_checksum_ok[0x1];
	u8 ip_ttl_hoplimit[0x8];
	u8 udp_sport[0x10];
	u8 udp_dport[0x10];
	u8 src_ip_127_96[0x20];
	u8 src_ip_95_64[0x20];
	u8 src_ip_63_32[0x20];
	u8 src_ip_31_0[0x20];
	u8 dst_ip_127_96[0x20];
	u8 dst_ip_95_64[0x20];
	u8 dst_ip_63_32[0x20];
	u8 dst_ip_31_0[0x20];
};

struct mlx5_ifc_dr_match_set_misc_bits {
	u8 gre_c_present[0x1];
	u8 bth_a[0x1];
	u8 gre_k_present[0x1];
	u8 gre_s_present[0x1];
	u8 source_vhca_port[0x4];
	u8 source_sqn[0x18];
	u8 source_eswitch_owner_vhca_id[0x10];
	u8 source_port[0x10];
	u8 outer_second_prio[0x3];
	u8 outer_second_cfi[0x1];
	u8 outer_second_vid[0xc];
	u8 inner_second_prio[0x3];
	u8 inner_second_cfi[0x1];
	u8 inner_second_vid[0xc];
	u8 outer_second_cvlan_tag[0x1];
	u8 inner_second_cvlan_tag[0x1];
	u8 outer_second_svlan_tag[0x1];
	u8 inner_second_svlan_tag[0x1];
	u8 outer_emd_tag[0x1];
	u8 reserved_at_65[0xb];
	u8 gre_protocol[0x10];
	u8 gre_key_h[0x18];
	u8 gre_key_l[0x8];
	u8 vxlan_vni[0x18];
	u8 bth_opcode[0x8];
	u8 geneve_vni[0x18];
	u8 reserved_at_e4[0x6];
	u8 geneve_tlv_option_0_exist[0x1];
	u8 geneve_oam[0x1];
	u8 reserved_at_ec[0xc];
	u8 outer_ipv6_flow_label[0x14];
	u8 reserved_at_100[0xc];
	u8 inner_ipv6_flow_label[0x14];
	u8 reserved_at_120[0xa];
	u8 geneve_opt_len[0x6];
	u8 geneve_protocol_type[0x10];
	u8 reserved_at_140[0x8];
	u8 bth_dst_qp[0x18];
	u8 inner_esp_spi[0x20];
	u8 outer_esp_spi[0x20];
	u8 reserved_at_1a0[0x20];
	u8 reserved_at_1c0[0x20];
	u8 reserved_at_1e0[0x20];
};

struct mlx5_ifc_dr_match_set_misc2_bits {
	union {
		u8 outer_first_mpls[0x20];
		struct {
			u8 outer_first_mpls_label[0x14];
			u8 outer_first_mpls_exp[0x3];
			u8 outer_first_mpls_s_bos[0x1];
			u8 outer_first_mpls_ttl[0x8];
		};
	};
	union {
		u8 inner_first_mpls[0x20];
		struct {
			u8 inner_first_mpls_label[0x14];
			u8 inner_first_mpls_exp[0x3];
			u8 inner_first_mpls_s_bos[0x1];
			u8 inner_first_mpls_ttl[0x8];
		};
	};
	u8 outer_first_mpls_over_gre_label[0x14];
	u8 outer_first_mpls_over_gre_exp[0x3];
	u8 outer_first_mpls_over_gre_s_bos[0x1];
	u8 outer_first_mpls_over_gre_ttl[0x8];
	u8 outer_first_mpls_over_udp_label[0x14];
	u8 outer_first_mpls_over_udp_exp[0x3];
	u8 outer_first_mpls_over_udp_s_bos[0x1];
	u8 outer_first_mpls_over_udp_ttl[0x8];
	u8 metadata_reg_c_7[0x20];
	u8 metadata_reg_c_6[0x20];
	u8 metadata_reg_c_5[0x20];
	u8 metadata_reg_c_4[0x20];
	u8 metadata_reg_c_3[0x20];
	u8 metadata_reg_c_2[0x20];
	u8 metadata_reg_c_1[0x20];
	u8 metadata_reg_c_0[0x20];
	u8 metadata_reg_a[0x20];
	u8 psp_syndrome[0x8];
	u8 reserved_at_1a8[0x8];
	u8 ipsec_syndrome[0x8];
	u8 ipsec_next_header[0x8];
	u8 reserved_at_260[0x20];
	u8 reserved_at_280[0x20];
};

struct mlx5_ifc_dr_match_set_misc3_bits {
	u8 inner_tcp_seq_num[0x20];
	u8 outer_tcp_seq_num[0x20];
	u8 inner_tcp_ack_num[0x20];
	u8 outer_tcp_ack_num[0x20];
	u8 reserved_at_80[0x8];
	u8 outer_vxlan_gpe_vni[0x18];
	u8 outer_vxlan_gpe_next_protocol[0x8];
	u8 outer_vxlan_gpe_flags[0x8];
	u8 reserved_at_b0[0x10];
	u8 icmp_header_data[0x20];
	u8 icmpv6_header_data[0x20];
	u8 icmp_type[0x8];
	u8 icmp_code[0x8];
	u8 icmpv6_type[0x8];
	u8 icmpv6_code[0x8];
	u8 geneve_tlv_option_0_data[0x20];
	u8 gtpu_teid[0x20];
	u8 gtpu_msg_type[0x8];
	u8 gtpu_msg_flags[0x8];
	u8 reserved_at_150[0x10];
	union {
		u8 gtpu_dw_2[0x20];
		struct {
			u8 gtpu_dw_2_seq_num[0x10];
			u8 gtpu_dw_2_pdu_num[0x8];
			u8 gtpu_dw_2_next_ext_hdr_type[0x8];
		};
	};
	union {
		u8 gtpu_first_ext_dw_0[0x20];
		struct {
			u8 gtpu_first_ext_dw_0_len[0x8];
			u8 gtpu_first_ext_dw_0_pdu_type[0x4];
			u8 gtpu_first_ext_dw_0_flags[0x4];
			u8 gtpu_first_ext_dw_0_qfi[0x8];
			u8 gtpu_first_ext_dw_0_reserved2[0x8];
		};
	};
	union {
		u8 gtpu_dw_0[0x20];
		struct {
			u8 gtpu_dw_0_version[0x3];
			u8 gtpu_dw_0_proto_type[0x1];
			u8 gtpu_dw_0_reserved1[0x1];
			u8 gtpu_dw_0_ext_hdr_flag[0x1];
			u8 gtpu_dw_0_seq_num_flag[0x1];
			u8 gtpu_dw_0_pdu_flag[0x1];
			u8 gtpu_dw_0_msg_type[0x8];
			u8 gtpu_dw_0_msg_len[0x10];
		};
	};
	u8 reserved_at_1c0[0x20];
};

struct mlx5_ifc_dr_match_set_misc4_bits {
	u8 prog_sample_field_value_0[0x20];
	u8 prog_sample_field_id_0[0x20];
	u8 prog_sample_field_value_1[0x20];
	u8 prog_sample_field_id_1[0x20];
	u8 prog_sample_field_value_2[0x20];
	u8 prog_sample_field_id_2[0x20];
	u8 prog_sample_field_value_3[0x20];
	u8 prog_sample_field_id_3[0x20];
	u8 prog_sample_field_value_4[0x20];
	u8 prog_sample_field_id_4[0x20];
	u8 prog_sample_field_value_5[0x20];
	u8 prog_sample_field_id_5[0x20];
	u8 prog_sample_field_value_6[0x20];
	u8 prog_sample_field_id_6[0x20];
	u8 prog_sample_field_value_7[0x20];
	u8 prog_sample_field_id_7[0x20];
};

struct mlx5_ifc_dr_match_set_misc5_bits {
	u8 macsec_tag_0[0x20];
	u8 macsec_tag_1[0x20];
	u8 macsec_tag_2[0x20];
	u8 macsec_tag_3[0x20];
	u8 tunnel_header_0[0x20];
	u8 tunnel_header_1[0x20];
	u8 tunnel_header_2[0x20];
	u8 tunnel_header_3[0x20];
	u8 reserved[0x100];
};

struct mlx5_ifc_dr_match_set_misc6_bits {
	u8 psp_header_0[0x20];
	u8 psp_header_1[0x20];
	u8 psp_header_2[0x20];
	u8 psp_header_3[0x20];
	u8 psp_header_4[0x20];
	u8 psp_header_5[0x20];
	u8 psp_header_6[0x20];
	u8 psp_header_7[0x20];
	u8 psp_header_8[0x20];
	u8 psp_header_9[0x20];
	u8 reserved[0xc0];
};

struct mlx5_ifc_dr_match_param_bits {
	struct mlx5_ifc_dr_match_spec_bits outer;
	struct mlx5_ifc_dr_match_set_misc_bits misc;
	struct mlx5_ifc_dr_match_spec_bits inner;
	struct mlx5_ifc_dr_match_set_misc2_bits misc2;
	struct mlx5_ifc_dr_match_set_misc3_bits misc3;
	struct mlx5_ifc_dr_match_set_misc4_bits misc4;
	struct mlx5_ifc_dr_match_set_misc5_bits misc5;
	struct mlx5_ifc_dr_match_set_misc6_bits misc6;
};

enum {
	MLX5_FLOW_TABLE_HASH_TYPE_CRC32,
};

struct mlx5_ifc_query_match_sample_info_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];
	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];
	u8 reserved_at_40[0x60];
	u8 sample_field_id[0x20];
	u8 reserved_at_c0[0x140];
};

struct mlx5_ifc_query_match_sample_info_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x40];
	u8 reserved_at_80[0x4];
	u8 modify_field_id[0xc];
	u8 ok_bit_format_select_dw[0x8];
	u8 field_format_select_dw[0x8];
	u8 reserved_at_a0[0x3];
	u8 ok_bit_offset[0x5];
	u8 reserved_at_a8[0x18];
	u8 reserved_at_c0[0x40];
};

struct mlx5_ifc_parse_graph_arc_bits {
	u8 start_inner_tunnel[0x1];
	u8 reserved_at_1[0x7];
	u8 arc_parse_graph_node[0x8];
	u8 compare_condition_value[0x10];
	u8 parse_graph_node_handle[0x20];
	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_parse_graph_flow_match_sample_bits {
	u8 flow_match_sample_en[0x1];
	u8 sample_group_valid[0x1];
	u8 tlv_option_sample[0x1];
	u8 reserved_at_3[0x1];
	u8 flow_match_sample_offset_mode[0x4];
	u8 reserved_at_5[0x8];
	u8 flow_match_sample_field_offset[0x10];
	u8 reserved_at_32[0x4];
	u8 flow_match_sample_field_offset_shift[0x4];
	u8 flow_match_sample_field_base_offset[0x8];
	u8 reserved_at_48[0x8];
	u8 sample_group_id[0x5];
	u8 flow_match_sample_tunnel_mode[0x3];
	u8 flow_match_sample_field_offset_mask[0x20];
	u8 flow_match_sample_field_id[0x20];
};

struct mlx5_ifc_parse_graph_flow_match_sample_ext_bits {
	u8 tlv_option_type[0x20];
	u8 reserved_at_32[0x20];
};

struct mlx5_ifc_parse_graph_flex_bits {
	u8 modify_field_select[0x40];
	u8 tlv_option_length_base_value[0x10];
	u8 tlv_option_length_field_offset[0x10];
	u8 header_length_base_value[0x10];
	u8 tlv_option_length_field_shift[0x4];
	u8 header_length_field_shift[0x4];
	u8 tlv_option_length_mode[0x4];
	u8 header_length_mode[0x4];
	u8 header_length_field_offset[0x10];
	u8 next_header_field_offset[0x10];
	u8 tlv_option_offset[0x10];
	u8 reserved_at_170[0x2];
	u8 head_anchor_id[0x6];
	u8 tlv_option_enable[0x1];
	u8 reserved_at_179[0x2];
	u8 next_header_field_size[0x5];
	u8 header_length_field_mask[0x20];
	u8 tlv_option_length_field_mask[0x20];
	struct mlx5_ifc_parse_graph_flow_match_sample_bits sample_table[0x8];
	struct mlx5_ifc_parse_graph_arc_bits input_arc[0x8];
	struct mlx5_ifc_parse_graph_arc_bits output_arc[0x8];
	u8 tlv_option_type_field_offset[0x10];
	u8 tlv_option_type_field_size[0x8];
	u8 reserved_at_412[0x8];
	u8 reserved_at_420[0x20];
	struct mlx5_ifc_parse_graph_flow_match_sample_ext_bits sample_ext_table[0x8];
};

struct mlx5_ifc_create_flex_parser_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_parse_graph_flex_bits flex;
};

struct mlx5_ifc_create_flex_parser_out_bits {
	struct mlx5_ifc_general_obj_out_cmd_hdr_bits hdr;
	struct mlx5_ifc_parse_graph_flex_bits flex;
};

struct mlx5_ifc_geneve_tlv_option_bits {
	u8 modify_field_select[0x40];
	u8 reserved_at_40[0x8];
	u8 sample_offset[0x8];
	u8 sample_id_valid[0x1];
	u8 sample_offset_valid[0x1];
	u8 option_class_ignore[0x1];
	u8 reserved_at_53[0x5];
	u8 geneve_option_fte_index[0x8];
	u8 option_class[0x10];
	u8 option_type[0x8];
	u8 reserved_at_78[0x3];
	u8 option_data_length[0x5];
	u8 geneve_sample_field_id[0x20];
	u8 reserved_at_a0[0x160];
};

struct mlx5_ifc_create_geneve_tlv_option_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_geneve_tlv_option_bits geneve_tlv_opt;
};

struct mlx5_ifc_query_geneve_tlv_option_out_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_geneve_tlv_option_bits geneve_tlv_opt;
};

#endif /* _HWS_MLX5_IFC_H_ */
