/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_PAT_AND_ARG_H_
#define _HWS_PAT_AND_ARG_H_

struct pattern_cache {
	/* Protect pattern list */
	pthread_spinlock_t lock;
	struct list_head head;
};

struct pattern_cache_item {
	struct {
		struct mlx5dv_devx_obj *pattern_obj;
		uint8_t *data;
		uint16_t num_of_actions;
	} mh_data;
	uint32_t refcount;
	struct list_node next;
};

bool arg_check_idx_aligned(uint32_t arg_idx, size_t data_sz);

int pat_cache_init(struct pattern_cache **cache);

void pat_cache_uninit(struct pattern_cache *cache);

bool pat_verify_actions(__be64 pattern[], size_t sz);

struct mlx5dv_devx_obj *
pat_get_pattern(struct mlx5dv_hws_context *ctx,
		__be64 *pattern,
		size_t pattern_sz);

void pat_get_pattern_by_pat_obj(struct mlx5dv_hws_context *ctx,
				struct mlx5dv_devx_obj *pat_obj);

void pat_put_pattern(struct mlx5dv_hws_context *ctx,
		     struct mlx5dv_devx_obj *pat_obj);

bool pat_require_reparse(__be64 *actions, uint16_t num_of_actions);

void arg_write(struct send_engine *queue,
	       void *comp_data,
	       uint32_t arg_idx,
	       uint8_t *arg_data,
	       size_t data_size);

void
arg_prepare_decap_l3_data(uint8_t *src, uint8_t *dst, uint16_t num_of_actions);

void arg_decapl3_write(struct send_engine *queue,
		       uint32_t arg_idx,
		       uint8_t *arg_data,
		       uint16_t num_of_actions);

#endif /* _HWS_PAT_AND_ARG_H_ */
