/* SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef _HWS_RULE_H_
#define _HWS_RULE_H_

enum rule_wqe_gta_sz {
	RULE_WQE_SZ_GTA_CTRL = 48,
	RULE_WQE_SZ_GTA_DATA = 64,
};

enum rule_match_tag_sz {
	RULE_MATCH_TAG_SZ_ACTION = 12,
	RULE_MATCH_TAG_SZ_MATCH = 32,
	RULE_MATCH_TAG_SZ_JUMBO = 44,
};

enum rule_status {
	RULE_STATUS_UNKNOWN,
	RULE_STATUS_CREATING,
	RULE_STATUS_CREATED,
	RULE_STATUS_DELETING,
	RULE_STATUS_DELETED,
	RULE_STATUS_FAILING,
	RULE_STATUS_FAILED,
};

enum rule_move_state {
	RULE_RESIZE_STATE_IDLE,
	RULE_RESIZE_STATE_WRITING,
	RULE_RESIZE_STATE_DELETING,
};
struct rule_match_tag {
	union {
		uint8_t jumbo[RULE_MATCH_TAG_SZ_JUMBO];
		struct {
			uint8_t reserved[RULE_MATCH_TAG_SZ_ACTION];
			uint8_t match[RULE_MATCH_TAG_SZ_MATCH];
		};
	};
};

struct rule_resize_info {
	struct pool *action_ste_pool;
	uint32_t rtc;
	uint32_t rule_idx;
	uint8_t state;
	uint8_t max_stes;
	/* Ctrl segment of STE: 48 bytes */
	uint8_t ctrl_seg[RULE_WQE_SZ_GTA_CTRL];
	/* Data segment of STE: 64 bytes */
	uint8_t data_seg[RULE_WQE_SZ_GTA_DATA];
};
struct mlx5dv_hws_rule {
	struct mlx5dv_hws_matcher *matcher;
	union {
		struct rule_match_tag tag;
		/* Pointer to tag to store more than one tag */
		struct rule_match_tag *tag_ptr;
		struct ibv_flow *flow;
		struct rule_resize_info *resize_info;
	};
	uint32_t rtc; /* The RTC into which the STE was inserted */
	uint32_t rtc_ext; /* The extended RTC into which the STE was inserted */
	int action_ste_idx; /* STE array index */
	uint8_t status; /* enum mlx5dv_hws_rule_status */
	uint8_t pending_wqes;
};

void rule_free_action_ste_idx(struct mlx5dv_hws_rule *rule);

int rule_move_hws_remove(struct mlx5dv_hws_rule *rule,
			 void *queue, void *user_data);

int rule_move_hws_add(struct mlx5dv_hws_rule *rule,
		      struct mlx5dv_hws_rule_attr *attr);

bool rule_move_in_progress(struct mlx5dv_hws_rule *rule);

void rule_clear_resize_info(struct mlx5dv_hws_rule *rule);

int rule_create_root_no_comp(struct mlx5dv_hws_rule *rule,
			     struct mlx5dv_hws_item_data item_data[],
			     struct mlx5dv_hws_action *dest_action,
			     uint8_t num_actions,
			     struct mlx5dv_hws_action_data actions_data[]);

int rule_destroy_root_no_comp(struct mlx5dv_hws_rule *rule);

void rule_destroy_failed_hws(struct mlx5dv_hws_rule *rule,
			     struct mlx5dv_hws_rule_attr *attr);

void rule_create_init_ste_attr(struct mlx5dv_hws_rule *rule,
			       struct send_ste_attr *ste_attr,
			       bool is_update);

void rule_init_dep_wqe(struct send_ring_dep_wqe *dep_wqe,
		       struct mlx5dv_hws_rule *rule,
		       struct mlx5dv_hws_rule_attr *attr,
		       bool is_update);

#endif /* _HWS_RULE_H_ */
